from typing import List

import strawberry
import strawberry_django

from .bgp.types import (
    BGPPeerTemplateType,
    BGPPolicyTemplateType,
    BGPSessionTemplateType,
    BGPRouterType,
    BGPScopeType,
    BGPAddressFamilyType,
    BGPPeerType,
    BGPPeerAddressFamilyType,
    BFDProfileType,
)
from .community.types import CommunityType, CommunityListType, CommunityListEntryType
from .eigrp.types import (
    EIGRPRouterType,
    EIGRPAddressFamilyType,
    EIGRPNetworkType,
    EIGRPInterfaceType,
)
from .objects.types import (
    ASPathType,
    ASPathEntryType,
    PrefixListType,
    PrefixListEntryType,
    RouteMapType,
    RouteMapEntryType,
)
from .ospf.types import OSPFInstanceType, OSPFAreaType, OSPFInterfaceType
from .static.types import StaticRouteType


@strawberry.type(name="Query")
class StaticRouteQuery:
    static_route: StaticRouteType = strawberry_django.field()
    static_route_list: List[StaticRouteType] = strawberry_django.field()


@strawberry.type(name="Query")
class OSPFQuery:
    ospf_instance: OSPFInstanceType = strawberry_django.field()
    ospf_instance_list: List[OSPFInstanceType] = strawberry_django.field()

    ospf_area: OSPFAreaType = strawberry_django.field()
    ospf_area_list: List[OSPFAreaType] = strawberry_django.field()

    ospf_interface: OSPFInterfaceType = strawberry_django.field()
    ospf_interface_list: List[OSPFInterfaceType] = strawberry_django.field()


@strawberry.type(name="Query")
class EIGRPQuery:
    eigrp_router: EIGRPRouterType = strawberry_django.field()
    eigrp_router_list: List[EIGRPRouterType] = strawberry_django.field()

    eigrp_address_family: EIGRPAddressFamilyType = strawberry_django.field()
    eigrp_address_family_list: List[EIGRPAddressFamilyType] = strawberry_django.field()

    eigrp_network: EIGRPNetworkType = strawberry_django.field()
    eigrp_network_list: List[EIGRPNetworkType] = strawberry_django.field()

    eigrp_interface: EIGRPInterfaceType = strawberry_django.field()
    eigrp_interface_list: List[EIGRPInterfaceType] = strawberry_django.field()


@strawberry.type(name="Query")
class CommunityQuery:
    community: CommunityType = strawberry_django.field()
    community_list: List[CommunityType] = strawberry_django.field()


@strawberry.type(name="Query")
class CommunityListQuery:
    communitylist: CommunityListType = strawberry_django.field()
    communitylist_list: List[CommunityListType] = strawberry_django.field()


@strawberry.type(name="Query")
class CommunityListEntryQuery:
    communitylistentry: CommunityListEntryType = strawberry_django.field()
    communitylistentry_list: List[CommunityListEntryType] = strawberry_django.field()


@strawberry.type(name="Query")
class ASPathQuery:
    aspath: ASPathType = strawberry_django.field()
    aspath_list: List[ASPathType] = strawberry_django.field()


@strawberry.type(name="Query")
class ASPathEntryQuery:
    aspath_entry: ASPathEntryType = strawberry_django.field()
    aspath_entry_list: List[ASPathEntryType] = strawberry_django.field()


@strawberry.type(name="Query")
class PrefixListQuery:
    prefixlist: PrefixListType = strawberry_django.field()
    prefixlist_list: List[PrefixListType] = strawberry_django.field()


@strawberry.type(name="Query")
class PrefixListEntryQuery:
    prefixlist_entry: PrefixListEntryType = strawberry_django.field()
    prefixlist_entry_list: List[PrefixListEntryType] = strawberry_django.field()


@strawberry.type(name="Query")
class RouteMapQuery:
    route_map: RouteMapType = strawberry_django.field()
    route_map_list: List[RouteMapType] = strawberry_django.field()


@strawberry.type(name="Query")
class RouteMapEntryQuery:
    route_map_entry: RouteMapEntryType = strawberry_django.field()
    route_map_entry_list: List[RouteMapEntryType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPPeerTemplateQuery:
    bgp_peer_template: BGPPeerTemplateType = strawberry_django.field()
    bgp_peer_template_list: List[BGPPeerTemplateType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPPolicyTemplateQuery:
    bgp_policy_template: BGPPolicyTemplateType = strawberry_django.field()
    bgp_policy_template_list: List[BGPPolicyTemplateType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPSessionTemplateQuery:
    bgp_session_template: BGPSessionTemplateType = strawberry_django.field()
    bgp_session_template_list: List[BGPSessionTemplateType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPRouterQuery:
    bgp_router: BGPRouterType = strawberry_django.field()
    bgp_router_list: List[BGPRouterType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPScopeQuery:
    bgp_scope: BGPScopeType = strawberry_django.field()
    bgp_scope_list: List[BGPScopeType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPAddressFamilyQuery:
    bgp_address_family: BGPAddressFamilyType = strawberry_django.field()
    bgp_address_family_list: List[BGPAddressFamilyType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPPeerQuery:
    bgp_peer: BGPPeerType = strawberry_django.field()
    bgp_peer_list: List[BGPPeerType] = strawberry_django.field()


@strawberry.type(name="Query")
class BGPPeerAddressFamilyQuery:
    bgp_peer_address_family: BGPPeerAddressFamilyType = strawberry_django.field()
    bgp_peer_address_family_list: List[BGPPeerAddressFamilyType] = (
        strawberry_django.field()
    )


@strawberry.type(name="Query")
class BFDProfileQuery:
    bfd_profile: BFDProfileType = strawberry_django.field()
    bfd_profile_list: List[BFDProfileType] = strawberry_django.field()


schema = [
    StaticRouteQuery,
    OSPFQuery,
    EIGRPQuery,
    CommunityQuery,
    CommunityListQuery,
    CommunityListEntryQuery,
    ASPathQuery,
    ASPathEntryQuery,
    PrefixListQuery,
    PrefixListEntryQuery,
    RouteMapQuery,
    RouteMapEntryQuery,
    BGPPeerTemplateQuery,
    BGPPolicyTemplateQuery,
    BGPSessionTemplateQuery,
    BGPRouterQuery,
    BGPScopeQuery,
    BGPAddressFamilyQuery,
    BGPPeerQuery,
    BGPPeerAddressFamilyQuery,
    BFDProfileQuery,
]



from netbox.search import SearchIndex, register_search
from netbox_routing.models.eigrp import EIGRPRouter, EIGRPAddressFamily

__all__ = (
    'EIGRPRouterIndex',
    'EIGRPAddressFamilyIndex',
)


@register_search
class EIGRPRouterIndex(SearchIndex):
    model = EIGRPRouter
    fields = (
        ('rid', 100),
        ('pid', 200),
        ('name', 210),
        ('comments', 5000),
    )
    display_attrs = ('device', 'mode')


@register_search
class EIGRPAddressFamilyIndex(SearchIndex):
    model = EIGRPAddressFamily
    fields = (
        ('rid', 100),
        ('comments', 5000),
    )



from django.db.models import Q

EIGRP_ROUTER_MODELS = Q(
    app_label='netbox_routing',
    model__in=(
        'eigrpnamedrouter',
        'eigrpclassicrouter',
    ),
)

EIGRP_ASSIGNABLE_MODELS = Q(
    app_label='netbox_routing',
    model__in=(
        'eigrpnamedrouter',
        'eigrpclassicrouter',
        'eigrpaddressfamily',
    ),
)



from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.core.validators import (
    MinValueValidator,
    MaxValueValidator,
)
from django.db.models.functions import Lower
from django.utils.translation import gettext_lazy as _
from django.urls import reverse

from django.db import models
from django.core.exceptions import ValidationError

from ipam.choices import IPAddressFamilyChoices
from ipam.fields import IPNetworkField
from netbox.models import PrimaryModel

from netbox_routing.models.community import *
from netbox_routing.choices import ActionChoices
from netbox_routing.constants.objects import PREFIX_ASSIGNMENT_MODELS

__all__ = (
    'ASPath',
    'ASPathEntry',
    'PrefixList',
    'PrefixListEntry',
    'CustomPrefix',
    'RouteMap',
    'RouteMapEntry',
)


class PermitDenyChoiceMixin:
    def get_action_color(self):
        return ActionChoices.colors.get(self.action)


class ASPath(PrimaryModel):
    name = models.CharField(max_length=100)

    clone_fields = ()
    prerequisite_models = ()

    class Meta:
        ordering = [
            'name',
        ]
        constraints = (
            models.UniqueConstraint(
                Lower('name'),
                name='%(app_label)s_%(class)s_unique_name',
                violation_error_message="Name must be unique.",
            ),
        )

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:aspath', args=[self.pk])


class ASPathEntry(PermitDenyChoiceMixin, PrimaryModel):
    aspath = models.ForeignKey(
        to=ASPath,
        on_delete=models.CASCADE,
        related_name='aspath_entries',
        verbose_name=_('AS-Path'),
    )
    sequence = models.PositiveSmallIntegerField()
    action = models.CharField(max_length=6, choices=ActionChoices)
    pattern = models.CharField(
        max_length=200,
    )

    clone_fields = (
        'aspath',
        'action',
    )
    prerequisite_models = ('netbox_routing.ASPath',)

    class Meta:
        ordering = ['aspath', 'sequence']
        constraints = (
            models.UniqueConstraint(
                fields=(
                    'aspath',
                    'sequence',
                ),
                name='%(app_label)s_%(class)s_unique_aspath_sequence',
                violation_error_message="Prefix List sequence must be unique.",
            ),
        )

    def __str__(self):
        # Workaround for bug with nulled aspath entry.
        if hasattr(self, 'aspath'):
            return f'{self.aspath.name} {self.action} {self.sequence}'
        return super().__str__()

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:aspathentry', args=[self.pk])


class PrefixList(PrimaryModel):
    name = models.CharField(max_length=100)
    family = models.PositiveSmallIntegerField(choices=IPAddressFamilyChoices, default=4)

    clone_fields = ()
    prerequisite_models = ()

    class Meta:
        ordering = [
            'name',
        ]
        constraints = (
            models.UniqueConstraint(
                Lower('name'),
                name='%(app_label)s_%(class)s_unique_name',
                violation_error_message="Name must be unique.",
            ),
        )

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:prefixlist', args=[self.pk])


class PrefixListEntry(PermitDenyChoiceMixin, PrimaryModel):
    prefix_list = models.ForeignKey(
        to=PrefixList,
        on_delete=models.CASCADE,
        related_name='prefix_list_entries',
        verbose_name='Prefix List',
    )
    assigned_prefix_type = models.ForeignKey(
        verbose_name=_('Prefix Type'),
        to=ContentType,
        limit_choices_to=PREFIX_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    assigned_prefix_id = models.PositiveBigIntegerField(
        verbose_name=_('Prefix ID'), blank=True, null=True
    )
    assigned_prefix = GenericForeignKey(
        ct_field='assigned_prefix_type',
        fk_field='assigned_prefix_id',
    )
    sequence = models.PositiveSmallIntegerField()
    action = models.CharField(max_length=6, choices=ActionChoices)
    ge = models.PositiveSmallIntegerField(
        verbose_name='GE',
        null=True,
        blank=True,
        validators=[MinValueValidator(0), MaxValueValidator(128)],
    )
    le = models.PositiveSmallIntegerField(
        verbose_name='LE',
        null=True,
        blank=True,
        validators=[MinValueValidator(0), MaxValueValidator(128)],
    )

    clone_fields = (
        'prefix_list',
        'action',
    )
    prerequisite_models = ('netbox_routing.PrefixList',)

    class Meta:
        ordering = ['prefix_list', 'sequence']
        constraints = (
            models.UniqueConstraint(
                fields=(
                    'prefix_list',
                    'sequence',
                ),
                name='%(app_label)s_%(class)s_unique_prefixlist_sequence',
                violation_error_message="Prefix List sequence must be unique.",
            ),
        )

    def __str__(self):
        return f'{self.prefix_list.name} {self.action} {self.sequence}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:prefixlistentry', args=[self.pk])

    def clean(self):
        super().clean()
        if self.assigned_prefix:
            prefix = self.assigned_prefix.prefix
            boundary = 32 if prefix.version == 4 else 128

            if self.le is not None and self.le > boundary:
                raise ValidationError({'le': 'LE value cannot be longer then 32'})
            if self.ge is not None and self.ge > boundary:
                raise ValidationError({'ge': 'GE value cannot be longer then 32'})

            if self.ge and self.le and self.ge < self.le:
                raise ValidationError(
                    {
                        'ge': 'GE cannot be more then LE',
                        'le': 'LE cannot be less then GE',
                    }
                )

            if self.ge is not None and prefix.prefixlen >= self.ge:
                raise ValidationError(
                    'Prefix\'s length cannot be longer then greater or equals value'
                )

            if self.le is not None and prefix.prefixlen >= self.le:
                raise ValidationError(
                    'Prefix\'s length cannot be longer then greater or equals value'
                )


class CustomPrefix(PrimaryModel):
    prefix = IPNetworkField(help_text='IPv4 or IPv6 network with mask')

    prefix_list_enties = GenericRelation(
        verbose_name=_('Settings'),
        to='netbox_routing.PrefixListEntry',
        related_name='custom_prefixes',
        related_query_name='custom_prefixes',
        content_type_field='assigned_prefix_type',
        object_id_field='assigned_prefix_id',
    )

    clone_fields = ('prefix',)
    prerequisite_models = ()

    class Meta:
        ordering = [
            'prefix',
        ]
        constraints = (
            models.UniqueConstraint(
                fields=('prefix',),
                name='%(app_label)s_%(class)s_unique_prefix',
                violation_error_message="Prefix must be unique.",
            ),
        )

    def __str__(self):
        return f'{self.prefix}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:customprefix', args=[self.pk])


class RouteMap(PrimaryModel):
    name = models.CharField(max_length=100)

    clone_fields = ()
    prerequisite_models = ()

    class Meta:
        ordering = [
            'name',
        ]
        constraints = (
            models.UniqueConstraint(
                Lower('name'),
                name='%(app_label)s_%(class)s_unique_name',
                violation_error_message="Name must be unique.",
            ),
        )

    def __str__(self):
        return f'{self.name}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:routemap', kwargs={'pk': self.pk})


class RouteMapEntry(PermitDenyChoiceMixin, PrimaryModel):
    route_map = models.ForeignKey(
        to=RouteMap,
        on_delete=models.CASCADE,
        related_name='route_map_entries',
        verbose_name='Route Map',
    )
    action = models.CharField(max_length=6, choices=ActionChoices)
    sequence = models.PositiveSmallIntegerField()
    flow_control = models.PositiveSmallIntegerField(
        verbose_name=_('Flow Control (Continue)'), null=True, blank=True
    )
    match_community_list = models.ManyToManyField(
        to=CommunityList,
        blank=True,
        related_name='route_map_entries',
    )
    match_community = models.ManyToManyField(
        to=Community,
        blank=True,
        related_name='route_map_entries',
    )
    match_aspath = models.ManyToManyField(
        to=ASPath,
        blank=True,
        related_name='route_map_entries',
    )
    match_prefix_list = models.ManyToManyField(
        to=PrefixList,
        blank=True,
        related_name='route_map_entries',
    )
    match = models.JSONField(
        blank=True,
        null=True,
        verbose_name=_('Match parameters'),
        help_text=_("JSON blob of options to match "),
    )
    set = models.JSONField(
        blank=True,
        null=True,
        verbose_name=_('Set parameters'),
        help_text=_("JSON blob of options to set "),
    )

    clone_fields = (
        'route_map',
        'action',
    )
    prerequisite_models = ('netbox_routing.RouteMap',)

    class Meta:
        ordering = ['route_map', 'sequence']
        constraints = (
            models.UniqueConstraint(
                fields=(
                    'route_map',
                    'sequence',
                ),
                name='%(app_label)s_%(class)s_unique_routemap_sequence',
                violation_error_message="Route Map sequence must be unique.",
            ),
        )

    def __str__(self):
        return f'{self.route_map.name} {self.action} {self.sequence}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:routemapentry', args=[self.pk])



from netbox.plugins import PluginMenuButton, PluginMenuItem

__all__ = ('eigrp',)


COL_ADD = 'mdi mdi-plus'
COL_IMPORT = 'mdi mdi-upload'


routers = PluginMenuItem(
    link='plugins:netbox_routing:eigrprouter_list',
    link_text='Routers',
    permissions=['netbox_routing.view_eigrprouter'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:eigrprouter_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_eigrprouter'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:eigrprouter_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_eigrprouter'],
        ),
    ),
)
address_families = PluginMenuItem(
    link='plugins:netbox_routing:eigrpaddressfamily_list',
    link_text='Address Families',
    permissions=['netbox_routing.view_eigrpaddressfamily'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:eigrpaddressfamily_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_eigrpaddressfamily'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:eigrpaddressfamily_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_eigrpaddressfamily'],
        ),
    ),
)
networks = PluginMenuItem(
    link='plugins:netbox_routing:eigrpnetwork_list',
    link_text='Networks',
    permissions=['netbox_routing.view_eigrpnetwork'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:eigrpnetwork_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_eigrpnetwork'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:eigrpnetwork_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_eigrpnetwork'],
        ),
    ),
)
interfaces = PluginMenuItem(
    link='plugins:netbox_routing:eigrpinterface_list',
    link_text='Interfaces',
    permissions=['netbox_routing.view_eigrpinterface'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:eigrpinterface_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_eigrpinterface'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:eigrpinterface_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_eigrpinterface'],
        ),
    ),
)

eigrp = (routers, address_families, networks, interfaces)



from django import forms
from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from ipam.models import VRF
from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.choices import AuthenticationChoices
from netbox_routing.choices.ospf import OSPFAreaTypeChoices
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, add_blank_choice
from utilities.forms.fields import TagFilterField, DynamicModelMultipleChoiceField

from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface

__all__ = (
    'OSPFAreaFilterForm',
    'OSPFInstanceFilterForm',
    'OSPFInterfaceFilterForm',
)

from utilities.forms.rendering import FieldSet


class OSPFInstanceFilterForm(NetBoxModelFilterSetForm):
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    vrf_id = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    router_id = forms.CharField(
        required=False,
        label=_('Router ID'),
    )
    process_id = forms.CharField(
        required=False,
        label=_('Process ID'),
    )

    model = OSPFInstance
    fieldsets = (
        FieldSet(
            'q',
            'filter_id',
            'tag',
        ),
        FieldSet('device_id', 'vrf_id', 'router_id', 'process_id', name=_('Device')),
    )
    tag = TagFilterField(model)


class OSPFAreaFilterForm(NetBoxModelFilterSetForm):
    model = OSPFArea
    fieldsets = (FieldSet('q', 'device_id', 'filter_id', 'area_type', 'tag'),)

    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    area_type = forms.ChoiceField(
        choices=add_blank_choice(OSPFAreaTypeChoices), label='Area Type', required=False
    )
    tag = TagFilterField(model)


class OSPFInterfaceFilterForm(NetBoxModelFilterSetForm):
    model = OSPFInterface
    fieldsets = (
        FieldSet('q', 'filter_id', 'tag'),
        FieldSet('instance_id', 'area_id', name=_('OSPF')),
        FieldSet('device_id', 'interface_id', 'vrf_id', name=_('Device')),
        FieldSet('priority', 'bfd', 'authentication', name=_('Attributes')),
    )
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    vrf_id = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    instance_id = DynamicModelMultipleChoiceField(
        queryset=OSPFInstance.objects.all(),
        required=False,
        selector=True,
        label=_('Instance'),
    )
    area_id = DynamicModelMultipleChoiceField(
        queryset=OSPFArea.objects.all(),
        required=False,
        selector=True,
        label=_('Area'),
    )
    interface_id = DynamicModelMultipleChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        selector=True,
        label=_('Interface'),
    )
    passive = forms.NullBooleanField(
        required=False,
        label='Passive Interface',
        widget=forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
    )
    bfd = forms.NullBooleanField(
        required=False,
        label='BFD Enabled',
        widget=forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
    )
    priority = forms.IntegerField(required=False)
    authentication = forms.ChoiceField(
        choices=add_blank_choice(AuthenticationChoices), required=False
    )
    tag = TagFilterField(model)



from netbox.plugins import PluginMenuButton, PluginMenuItem

__all__ = ('MENUITEMS',)

COL_ADD = 'mdi mdi-plus'
COL_IMPORT = 'mdi mdi-upload'

ospf_instance = PluginMenuItem(
    link='plugins:netbox_routing:ospfinstance_list',
    link_text='Instances',
    permissions=['netbox_routing.view_ospfinstance'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:ospfinstance_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_ospfinstance'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:ospfinstance_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_ospfinstance'],
        ),
    ),
)
ospf_area = PluginMenuItem(
    link='plugins:netbox_routing:ospfarea_list',
    link_text='Areas',
    permissions=['netbox_routing.view_ospfarea'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:ospfarea_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_ospfarea'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:ospfarea_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_ospfarea'],
        ),
    ),
)
ospf_interfaces = PluginMenuItem(
    link='plugins:netbox_routing:ospfinterface_list',
    link_text='Interfaces',
    permissions=['netbox_routing.view_ospfinterface'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:ospfinterface_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_ospfinterface'],
        ),
        PluginMenuButton(
            link='plugins:netbox_routing:ospfinterface_bulk_import',
            title='Import',
            icon_class=COL_IMPORT,
            permissions=['netbox_routing.import_ospfinterface'],
        ),
    ),
)

MENUITEMS = (ospf_instance, ospf_area, ospf_interfaces)



# Generated by Django 5.2.5 on 2026-01-07 05:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0016_ospfinterface_interface_onetoonefield'),
        ('users', '0015_owner'),
    ]

    operations = [
        migrations.AddField(
            model_name='bgpaddressfamily',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgppeer',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgppeeraddressfamily',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgppeertemplate',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgppoliytemplate',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgprouter',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgpscope',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='bgpsetting',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='eigrpaddressfamily',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='eigrpinterface',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='eigrpnetwork',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='eigrprouter',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='ospfarea',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='ospfinstance',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='ospfinterface',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='prefixlist',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='prefixlistentry',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='routemap',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
        migrations.AddField(
            model_name='staticroute',
            name='owner',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to='users.owner',
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-05-18 13:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0003_model_ordering_and_constraints'),
    ]

    operations = [
        migrations.AlterField(
            model_name='prefixlistentry',
            name='ge',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='prefixlistentry',
            name='le',
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]



from dcim.models import Device
from netbox.views import generic
from utilities.views import register_model_view, ViewTab

from netbox_routing.filtersets import StaticRouteFilterSet
from netbox_routing.models import StaticRoute
from netbox_routing.tables.static import StaticRouteTable


@register_model_view(Device, name='staticroutes', path='static_routes')
class DeviceStaticRoutesView(generic.ObjectChildrenView):
    template_name = 'generic/object_children.html'
    queryset = Device.objects.all()
    child_model = StaticRoute
    table = StaticRouteTable
    filterset = StaticRouteFilterSet
    tab = ViewTab(
        label='Static Routes',
        badge=lambda obj: StaticRoute.objects.filter(devices=obj).count(),
        permission='netbox_routing.view_staticroute',
    )

    def get_children(self, request, device):
        return self.child_model.objects.filter(devices=device)



from netbox import object_actions
from netbox.views.generic import (
    ObjectListView,
    ObjectView,
    ObjectEditView,
    ObjectDeleteView,
    ObjectChildrenView,
    BulkDeleteView,
    BulkEditView,
)
from netbox_routing.models import Community, CommunityList
from utilities.views import register_model_view, ViewTab, GetRelatedModelsMixin
from netbox_routing.filtersets.objects import *
from netbox_routing.forms.objects import *
from netbox_routing.models.objects import *
from netbox_routing.tables.objects import *


#
# Prefix List
#
@register_model_view(PrefixList, name='list', path='', detail=False)
class PrefixListListView(ObjectListView):
    queryset = PrefixList.objects.all()
    table = PrefixListTable
    filterset = PrefixListFilterSet
    filterset_form = PrefixListFilterForm


@register_model_view(PrefixList)
class PrefixListView(ObjectView):
    queryset = PrefixList.objects.all()
    template_name = 'netbox_routing/prefixlist.html'


@register_model_view(PrefixList, name='entries')
class PrefixListEntriesView(ObjectChildrenView):
    template_name = 'netbox_routing/objectchildrentable.html'
    queryset = PrefixList.objects.all()
    child_model = PrefixListEntry
    table = PrefixListEntryTable
    filterset = PrefixListEntryFilterSet
    tab = ViewTab(
        label='Entries',
        badge=lambda obj: PrefixListEntry.objects.filter(prefix_list=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(prefix_list=parent)

    def get_extra_context(self, request, instance):

        return {
            'url_parent': 'plugins:netbox_routing:prefixlist_entries',
            'url_add': 'plugins:netbox_routing:prefixlistentry_add',
            'url_bulk_edit': 'plugins:netbox_routing:prefixlistentry_bulk_edit',
            'url_bulk_delete': 'plugins:netbox_routing:prefixlistentry_bulk_delete',
            'parent_name': 'prefix_list',
            'parent_view': 'prefixlist',
            'parent_view_path': 'entries',
            'view': 'prefixlistentry',
        }


@register_model_view(PrefixList, name='add', detail=False)
@register_model_view(PrefixList, name='edit')
class PrefixListEditView(ObjectEditView):
    queryset = PrefixList.objects.all()
    form = PrefixListForm


@register_model_view(PrefixList, name='delete')
class PrefixListDeleteView(ObjectDeleteView):
    queryset = PrefixList.objects.all()


@register_model_view(PrefixList, name='bulk_edit', detail=False)
class PrefixListBulkEditView(BulkEditView):
    queryset = PrefixList.objects.all()
    filterset = PrefixListFilterSet
    form = PrefixListBulkEditForm
    table = PrefixListTable


@register_model_view(PrefixList, name='bulk_delete', detail=False)
class PrefixListBulkDeleteView(BulkDeleteView):
    queryset = PrefixList.objects.all()
    filterset = PrefixListFilterSet
    table = PrefixListTable


#
# Prefix List Entry
#


@register_model_view(PrefixListEntry, name='list', path='', detail=False)
class PrefixListEntryListView(ObjectListView):
    queryset = PrefixListEntry.objects.all()
    table = PrefixListEntryTable
    filterset = PrefixListEntryFilterSet
    filterset_form = PrefixListEntryFilterForm


@register_model_view(PrefixListEntry)
class PrefixListEntryView(ObjectView):
    queryset = PrefixListEntry.objects.all()
    template_name = 'netbox_routing/prefixlistentry.html'


@register_model_view(PrefixListEntry, name='add', detail=False)
@register_model_view(PrefixListEntry, name='edit')
class PrefixListEntryEditView(ObjectEditView):
    queryset = PrefixListEntry.objects.all()
    form = PrefixListEntryForm


@register_model_view(PrefixListEntry, name='delete')
class PrefixListEntryDeleteView(ObjectDeleteView):
    queryset = PrefixListEntry.objects.all()


@register_model_view(PrefixListEntry, name='bulk_edit', detail=False)
class PrefixListEntryBulkEditView(BulkEditView):
    queryset = PrefixListEntry.objects.all()
    filterset = PrefixListEntryFilterSet
    table = PrefixListEntryTable
    form = PrefixListEntryBulkEditForm


@register_model_view(PrefixListEntry, name='bulk_delete', detail=False)
class PrefixListEntryBulkDeleteView(BulkDeleteView):
    queryset = PrefixListEntry.objects.all()
    filterset = PrefixListEntryFilterSet
    table = PrefixListEntryTable


#
# Custom Prefix
#


@register_model_view(CustomPrefix, name='list', path='', detail=False)
class CustomPrefixListView(ObjectListView):
    queryset = CustomPrefix.objects.all()
    table = CustomPrefixTable
    filterset = CustomPrefixFilterSet
    filterset_form = CustomPrefixFilterForm
    actions = (object_actions.AddObject,)


@register_model_view(CustomPrefix)
class CustomPrefixView(ObjectView):
    queryset = CustomPrefix.objects.all()
    template_name = 'netbox_routing/customprefix.html'


@register_model_view(CustomPrefix, name='add', detail=False)
@register_model_view(CustomPrefix, name='edit')
class CustomPrefixEditView(ObjectEditView):
    queryset = CustomPrefix.objects.all()
    form = CustomPrefixForm


@register_model_view(CustomPrefix, name='delete')
class CustomPrefixDeleteView(ObjectDeleteView):
    queryset = CustomPrefix.objects.all()


#
# Route Map
#
@register_model_view(RouteMap, name='list', path='', detail=False)
class RouteMapListView(ObjectListView):
    queryset = RouteMap.objects.all()
    table = RouteMapTable
    filterset = RouteMapFilterSet
    filterset_form = RouteMapFilterForm


@register_model_view(RouteMap)
class RouteMapView(ObjectView):
    queryset = RouteMap.objects.all()
    template_name = 'netbox_routing/routemap.html'


@register_model_view(RouteMap, name='entries')
class RouteMapEntriesView(ObjectChildrenView):
    template_name = 'netbox_routing/objectchildrentable.html'
    queryset = RouteMap.objects.all()
    child_model = RouteMapEntry
    table = RouteMapEntryTable
    filterset = RouteMapEntryFilterSet
    tab = ViewTab(
        label='Entries',
        badge=lambda obj: RouteMapEntry.objects.filter(route_map=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(route_map=parent)

    def get_extra_context(self, request, instance):

        return {
            'url_parent': 'plugins:netbox_routing:routemap_entries',
            'url_add': 'plugins:netbox_routing:routemapentry_add',
            'url_bulk_edit': 'plugins:netbox_routing:routemapentry_bulk_edit',
            'url_bulk_delete': 'plugins:netbox_routing:routemapentry_bulk_delete',
            'parent_name': 'route_map',
            'parent_view': 'routemap',
            'parent_view_path': 'entries',
            'view': 'routemapentry',
        }


@register_model_view(RouteMap, name='add', detail=False)
@register_model_view(RouteMap, name='edit')
class RouteMapEditView(ObjectEditView):
    queryset = RouteMap.objects.all()
    form = RouteMapForm


@register_model_view(RouteMap, name='delete')
class RouteMapDeleteView(ObjectDeleteView):
    queryset = RouteMap.objects.all()


@register_model_view(RouteMap, name='bulk_edit', detail=False)
class RouteMapBulkEditView(BulkEditView):
    queryset = RouteMap.objects.all()
    filterset = RouteMapFilterSet
    form = RouteMapBulkEditForm
    table = RouteMapTable


@register_model_view(RouteMap, name='bulk_delete', detail=False)
class RouteMapBulkDeleteView(BulkDeleteView):
    queryset = RouteMap.objects.all()
    filterset = RouteMapFilterSet
    table = RouteMapTable


#
# Route Map Entry
#
@register_model_view(RouteMapEntry, name='list', path='', detail=False)
class RouteMapEntryListView(ObjectListView):
    queryset = RouteMapEntry.objects.all()
    table = RouteMapEntryTable
    filterset = RouteMapEntryFilterSet
    filterset_form = RouteMapEntryFilterForm


@register_model_view(RouteMapEntry)
class RouteMapEntryView(GetRelatedModelsMixin, ObjectView):
    queryset = RouteMapEntry.objects.all()
    template_name = 'netbox_routing/routemapentry.html'

    def get_extra_context(self, request, instance):

        return {
            'related_models': self.get_related_models(
                request,
                instance,
                omit='route_map',
                extra=(
                    (
                        PrefixList.objects.restrict(request.user, 'view').filter(
                            route_map_entries=instance
                        ),
                        'route_map_entry_id',
                    ),
                    (
                        CommunityList.objects.restrict(request.user, 'view').filter(
                            route_map_entries=instance
                        ),
                        'route_map_entry_id',
                    ),
                    (
                        Community.objects.restrict(request.user, 'view').filter(
                            route_map_entries=instance
                        ),
                        'route_map_entry_id',
                    ),
                    (
                        ASPath.objects.restrict(request.user, 'view').filter(
                            route_map_entries=instance
                        ),
                        'route_map_entry_id',
                    ),
                ),
            ),
        }


@register_model_view(RouteMapEntry, name='add', detail=False)
@register_model_view(RouteMapEntry, name='edit')
class RouteMapEntryEditView(ObjectEditView):
    queryset = RouteMapEntry.objects.all()
    form = RouteMapEntryForm


@register_model_view(RouteMapEntry, name='delete')
class RouteMapEntryDeleteView(ObjectDeleteView):
    queryset = RouteMapEntry.objects.all()


@register_model_view(RouteMapEntry, name='bulk_edit', detail=False)
class RouteMapEntryBulkEditView(BulkEditView):
    queryset = RouteMapEntry.objects.all()
    filterset = RouteMapEntryFilterSet
    table = RouteMapEntryTable
    form = RouteMapEntryBulkEditForm


@register_model_view(RouteMapEntry, name='bulk_delete', detail=False)
class RouteMapEntryBulkDeleteView(BulkDeleteView):
    queryset = RouteMapEntry.objects.all()
    filterset = RouteMapEntryFilterSet
    table = RouteMapEntryTable


#
# AS Path
#
@register_model_view(ASPath, name='list', path='', detail=False)
class ASPathListView(ObjectListView):
    queryset = ASPath.objects.all()
    filterset = ASPathFilterSet
    filterset_form = ASPathFilterForm
    table = ASPathTable


@register_model_view(ASPath)
class ASPathView(ObjectView):
    queryset = ASPath.objects.all()
    template_name = 'netbox_routing/aspath.html'


@register_model_view(ASPath, name='add', detail=False)
@register_model_view(ASPath, name='edit')
class ASPathEditView(ObjectEditView):
    queryset = ASPath.objects.all()
    form = ASPathForm


@register_model_view(ASPath, name='delete')
class ASPathDeleteView(ObjectDeleteView):
    queryset = ASPath.objects.all()


@register_model_view(ASPath, name='bulk_edit', detail=False)
class ASPathBulkEditView(BulkEditView):
    queryset = ASPath.objects.all()
    filterset = ASPathFilterSet
    form = ASPathBulkEditForm
    table = ASPathTable


@register_model_view(ASPath, name='bulk_delete', detail=False)
class ASPathBulkDeleteView(BulkDeleteView):
    queryset = ASPath.objects.all()
    filterset = ASPathFilterSet
    table = ASPathTable


@register_model_view(ASPath, name='entries')
class ASPathEntriesView(ObjectChildrenView):
    template_name = 'netbox_routing/objectchildrentable.html'
    queryset = ASPath.objects.all()
    child_model = ASPathEntry
    table = ASPathEntryTable
    filterset = ASPathEntryFilterSet
    tab = ViewTab(
        label='Entries',
        badge=lambda obj: ASPathEntry.objects.filter(aspath=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(aspath=parent)

    def get_extra_context(self, request, instance):
        print(self.actions)

        return {
            'url_parent': 'plugins:netbox_routing:aspath_entries',
            'url_add': 'plugins:netbox_routing:aspathentry_add',
            'url_bulk_edit': 'plugins:netbox_routing:aspathentry_bulk_edit',
            'url_bulk_delete': 'plugins:netbox_routing:aspathentry_bulk_delete',
            'parent_name': 'aspath',
            'parent_view': 'aspath',
            'parent_view_path': 'entries',
            'view': 'aspathentry',
        }


#
# Route Map Entry
#
@register_model_view(ASPathEntry, name='list', path='', detail=False)
class ASPathEntryListView(ObjectListView):
    queryset = ASPathEntry.objects.all()
    table = ASPathEntryTable
    filterset = ASPathEntryFilterSet
    filterset_form = ASPathEntryFilterForm


@register_model_view(ASPathEntry)
class ASPathEntryView(ObjectView):
    queryset = ASPathEntry.objects.all()
    template_name = 'netbox_routing/aspathentry.html'


@register_model_view(ASPathEntry, name='add', detail=False)
@register_model_view(ASPathEntry, name='edit')
class ASPathEntryEditView(ObjectEditView):
    queryset = ASPathEntry.objects.all()
    form = ASPathEntryForm


@register_model_view(ASPathEntry, name='delete')
class ASPathEntryDeleteView(ObjectDeleteView):
    queryset = ASPathEntry.objects.all()


@register_model_view(ASPathEntry, name='bulk_edit', detail=False)
class ASPathEntryBulkEditView(BulkEditView):
    queryset = ASPathEntry.objects.all()
    filterset = ASPathEntryFilterSet
    table = ASPathEntryTable
    form = ASPathEntryBulkEditForm


@register_model_view(ASPathEntry, name='bulk_delete', detail=False)
class ASPathEntryBulkDeleteView(BulkDeleteView):
    queryset = ASPathEntry.objects.all()
    filterset = ASPathEntryFilterSet
    table = ASPathEntryTable



from netbox.plugins import PluginMenuButton, PluginMenuItem

__all__ = ('OBJECT_MENU',)


COL_ADD = 'mdi mdi-plus'
COL_IMPORT = 'mdi mdi-upload'


customprefix = PluginMenuItem(
    link='plugins:netbox_routing:customprefix_list',
    link_text='Custom Prefixes',
    permissions=['netbox_routing.view_customprefix'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:customprefix_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_customprefix'],
        ),
    ),
)
prefixlist = PluginMenuItem(
    link='plugins:netbox_routing:prefixlist_list',
    link_text='Prefix Lists',
    permissions=['netbox_routing.view_prefixlist'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:prefixlist_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_prefixlist'],
        ),
        # PluginMenuButton(
        #     link='plugins:netbox_routing:prefixlist_bulk_import',
        #     title='Import',
        #     icon_class=COL_IMPORT,
        #     permissions=['netbox_routing.add_prefixlist'],
        # ),
    ),
)
prefixlist_entry = PluginMenuItem(
    link='plugins:netbox_routing:prefixlistentry_list',
    link_text='Prefix List Entries',
    permissions=['netbox_routing.view_prefixlistentry'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:prefixlistentry_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_prefixlistentry'],
        ),
    ),
)
routemap = PluginMenuItem(
    link='plugins:netbox_routing:routemap_list',
    link_text='Route Maps',
    permissions=['netbox_routing.view_routemap'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:routemap_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_routemap'],
        ),
    ),
)
routemap_entry = PluginMenuItem(
    link='plugins:netbox_routing:routemapentry_list',
    link_text='Route Map Entries',
    permissions=['netbox_routing.view_routemapentry'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:routemapentry_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_routemapentry'],
        ),
    ),
)
aspath = PluginMenuItem(
    link='plugins:netbox_routing:aspath_list',
    link_text='AS Path',
    permissions=['netbox_routing.view_aspath'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:aspath_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_aspath'],
        ),
    ),
)
aspath_entry = PluginMenuItem(
    link='plugins:netbox_routing:aspathentry_list',
    link_text='AS Path Entries',
    permissions=['netbox_routing.view_aspathentry'],
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_routing:aspathentry_add',
            title='Add',
            icon_class=COL_ADD,
            permissions=['netbox_routing.add_aspathentry'],
        ),
        # PluginMenuButton(
        #     link='plugins:netbox_routing:aspathentry_bulk_import',
        #     title='Import',
        #     icon_class=COL_IMPORT,
        #     permissions=['netbox_routing.add_aspathentry'],
        # ),
    ),
)

OBJECT_MENU = (
    customprefix,
    prefixlist,
    prefixlist_entry,
    routemap,
    routemap_entry,
    aspath,
    aspath_entry,
)



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext as _

from ipam.choices import IPAddressFamilyChoices
from netbox.models import PrimaryModel
from netbox_routing import choices
from netbox_routing.choices.eigrp import EIGRPRouterChoices
from netbox_routing.fields.ip import IPAddressField

__all__ = (
    'EIGRPRouter',
    'EIGRPAddressFamily',
    'EIGRPNetwork',
    'EIGRPInterface',
)


class EIGRPRouter(PrimaryModel):
    device = models.ForeignKey(
        verbose_name=_('Device'),
        to='dcim.Device',
        related_name='eigrp',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    mode = models.CharField(
        verbose_name=_('Mode'), max_length=10, choices=EIGRPRouterChoices
    )
    name = models.CharField(
        verbose_name=_('Name'), max_length=100, blank=True, null=True
    )
    pid = models.PositiveIntegerField(
        verbose_name=_('Process ID'), blank=True, null=True
    )
    rid = IPAddressField(verbose_name=_('Router ID'), blank=True, null=True)

    clone_fields = ('device', 'mode', 'name', 'pid', 'rid')

    prerequisite_models = ('dcim.Device',)

    class Meta:
        verbose_name = 'EIGRP Router'

    def __str__(self):
        if self.pid:
            return f'Process {self.pid} ({self.rid})'
        return f'{self.name} ({self.rid})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:eigrprouter', args=[self.pk])

    @property
    def identifier(self):
        if self.name:
            return f'{self.name}'
        elif self.pid:
            return f'{self.pid}'
        return f'{self}'


class EIGRPAddressFamily(PrimaryModel):
    router = models.ForeignKey(
        verbose_name=_('Router'),
        to=EIGRPRouter,
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    vrf = models.ForeignKey(
        verbose_name=_('VRF'),
        to='ipam.VRF',
        related_name='eigrp_address_families',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
    family = models.PositiveSmallIntegerField(
        verbose_name=_('Address Family'),
        choices=IPAddressFamilyChoices,
        blank=False,
        null=False,
    )
    rid = IPAddressField(verbose_name=_('Router ID'), blank=True, null=True)

    clone_fields = ('router', 'vrf', 'family')

    prerequisite_models = ('netbox_routing.EIGRPRouter',)

    class Meta:
        verbose_name = 'EIGRP Address Family'
        constraints = (
            models.UniqueConstraint(
                fields=('router', 'vrf', 'family'),
                name='%(app_label)s_%(class)s_unique_af',
            ),
        )

    def __str__(self):
        if self.vrf:
            return f'{self.router.identifier} ({self.family} vrf {self.vrf.name})'
        return f'{self.router.identifier} ({self.family})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:eigrpaddressfamily', args=[self.pk])


class EIGRPNetwork(PrimaryModel):
    router = models.ForeignKey(
        verbose_name=_('Router'),
        to=EIGRPRouter,
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    address_family = models.ForeignKey(
        verbose_name=_('Address Family'),
        to=EIGRPAddressFamily,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
    network = models.ForeignKey(
        verbose_name=_('Network'),
        to='ipam.Prefix',
        related_name='eigrp',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )

    clone_fields = ('router', 'address_family', 'network')

    prerequisite_models = (
        'netbox_routing.EIGRPRouter',
        'ipam.Prefix',
    )

    class Meta:
        verbose_name = 'EIGRP Network'
        constraints = (
            models.UniqueConstraint(
                fields=(
                    'router',
                    'address_family',
                    'network',
                ),
                name='%(app_label)s_%(class)s_unique_network',
            ),
        )

    def __str__(self):
        if self.address_family:
            return f'{self.network} ({self.router.identifier} {self.address_family})'
        return f'{self.network} ({self.router.identifier})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:eigrpnetwork', args=[self.pk])


class EIGRPInterface(PrimaryModel):
    router = models.ForeignKey(
        verbose_name=_('Router'),
        to=EIGRPRouter,
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    address_family = models.ForeignKey(
        verbose_name=_('Address Family'),
        to=EIGRPAddressFamily,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
    interface = models.ForeignKey(
        verbose_name=_('Interface'),
        to='dcim.Interface',
        related_name='eigrp',
        on_delete=models.CASCADE,
        blank=False,
        null=False,
    )
    passive = models.BooleanField(
        verbose_name=_('Passive'),
    )
    bfd = models.BooleanField(
        verbose_name=_('BFD'),
    )
    authentication = models.CharField(
        verbose_name=_('Authentication'),
        max_length=50,
        choices=choices.AuthenticationChoices,
        blank=True,
        null=True,
    )
    passphrase = models.CharField(
        verbose_name=_('Passphrase'), max_length=200, blank=True, null=True
    )

    clone_fields = (
        'router',
        'address_family',
        'interface',
        'passive',
        'bfd',
        'authentication',
        'passphrase',
    )

    prerequisite_models = (
        'netbox_routing.EIGRPRouter',
        'dcim.Interface',
    )

    class Meta:
        verbose_name = 'EIGRP Interface'
        constraints = (
            models.UniqueConstraint(
                fields=(
                    'router',
                    'address_family',
                    'interface',
                ),
                name='%(app_label)s_%(class)s_unique_interface',
            ),
        )

    def __str__(self):
        if self.address_family:
            return f'{self.interface.name} ({self.router.identifier} {self.address_family})'
        return f'{self.interface.name} ({self.router.identifier})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:eigrpinterface', args=[self.pk])



from ipam.models import VRF
from dcim.models import Device
from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.models import StaticRoute
from utilities.forms.fields import DynamicModelMultipleChoiceField, TagFilterField
from django.utils.translation import gettext as _

from utilities.forms.rendering import FieldSet
from django import forms


class StaticRouteFilterForm(NetBoxModelFilterSetForm):
    model = StaticRoute
    fieldsets = (
        FieldSet('q', 'filter_id', 'tag'),
        FieldSet(
            'device_id',
            'vrf_id',
            'prefix',
            'next_hop',
            'name',
            name=_('Route Attributes'),
        ),
    )
    vrf_id = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    prefix = forms.CharField(
        required=False,
        label=_('Prefix'),
    )

    next_hop = forms.CharField(
        required=False,
        label=_('Next Hop'),
    )

    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )

    name = forms.CharField(
        required=False,
        label=_('Name'),
    )
    tag = TagFilterField(model)



from django.utils.translation import gettext as _

from dcim.models import Device
from ipam.models import VRF
from netbox.forms import NetBoxModelForm
from netbox_routing.models import StaticRoute
from utilities.forms.fields import (
    DynamicModelChoiceField,
    DynamicModelMultipleChoiceField,
    CommentField,
)
from utilities.forms.rendering import FieldSet


class StaticRouteForm(NetBoxModelForm):
    devices = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        label=_('Devices'),
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        label=_('VRF'),
    )
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'devices',
            'vrf',
        ),
        FieldSet(
            'prefix',
            'next_hop',
            'interface_next_hop',
            'metric',
            name=_('Route'),
        ),
        FieldSet(
            'name',
            'description',
            'tag',
            'permanent',
            name=_('Metadata'),
        ),
    )

    class Meta:
        model = StaticRoute
        fields = (
            'devices',
            'vrf',
            'prefix',
            'next_hop',
            'interface_next_hop',
            'name',
            'metric',
            'permanent',
            'tag',
            'description',
            'comments',
            'tags',
        )

    def __init__(self, data=None, instance=None, *args, **kwargs):
        super().__init__(data=data, instance=instance, *args, **kwargs)

        if self.instance and self.instance.pk is not None:
            self.fields['devices'].initial = self.instance.devices.all().values_list(
                'id', flat=True
            )

    def save(self, *args, **kwargs):
        instance = super().save(*args, **kwargs)
        instance.devices.set(self.cleaned_data['devices'])
        return instance



from django.db import models
from django.db.models import CheckConstraint, Q
from django.urls import reverse
from django.utils.translation import gettext as _
from django.core.exceptions import ValidationError

from ipam.fields import IPNetworkField
from netbox.models import PrimaryModel
from netbox_routing.fields.ip import IPAddressField

__all__ = ('StaticRoute',)


class StaticRoute(PrimaryModel):
    devices = models.ManyToManyField(to='dcim.Device', related_name='static_routes')
    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        related_name='staticroutes',
        blank=True,
        null=True,
        verbose_name='VRF',
    )
    prefix = IPNetworkField(
        help_text=_('IPv4 or IPv6 network with mask'),
    )
    next_hop = IPAddressField(
        verbose_name=_('Next Hop'),
        blank=True,
        null=True,
    )
    interface_next_hop = models.CharField(
        verbose_name='Interface Next Hop',
        help_text='Forwarding interface, for routes without IP address as next hop',
        blank=True,
        null=True,
    )
    name = models.CharField(
        max_length=50,
        verbose_name='Name',
        blank=True,
        null=True,
        help_text='Optional name for this static route',
    )
    metric = models.PositiveSmallIntegerField(
        verbose_name='Metric',
        blank=True,
        default=1,
    )
    permanent = models.BooleanField(
        default=False,
        blank=True,
        null=True,
    )

    tag = models.IntegerField(
        verbose_name='Route Tag',
        help_text='Optional tag for this static route',
        blank=True,
        null=True,
    )

    clone_fields = (
        'name',
        'devices',
        'prefix',
        'next_hop',
        'vrf',
        'metric',
        'permanent',
    )

    prerequisite_models = ('dcim.Device',)

    class Meta:
        ordering = ['vrf', 'prefix', 'metric']
        constraints = (
            CheckConstraint(
                check=Q(Q(metric__lte=255) & Q(metric__gte=0)), name='metric_gte_lte'
            ),
            models.UniqueConstraint(
                'vrf',
                'prefix',
                'next_hop',
                name='%(app_label)s_%(class)s_unique_vrf_prefix_nexthop',
                violation_error_message="VRF, Prefix and Next Hop must be unique.",
            ),
        )

    def __str__(self):
        if self.next_hop:
            if self.vrf:
                return f'{self.prefix} VRF {self.vrf} next-hop {self.next_hop}'
            return f'{self.prefix} next-hop {self.next_hop}'
        elif self.vrf:
            return f'{self.prefix} VRF {self.vrf} next-hop {self.interface_next_hop}'
        return f'{self.prefix} next-hop {self.next_hop}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:staticroute', args=[self.pk])

    def clean(self):
        super().clean()
        if not self.interface_next_hop and not self.next_hop:
            raise ValidationError(
                {
                    "next_hop": "A route requires set either an IP next hop or an Interface next hop."
                }
            )



from netbox.object_actions import CloneObject, EditObject, DeleteObject
from netbox.views.generic import (
    ObjectView,
    ObjectEditView,
    ObjectListView,
    ObjectDeleteView,
    ObjectChildrenView,
    BulkEditView,
    BulkDeleteView,
)

from utilities.views import register_model_view, ViewTab

from netbox_routing.filtersets.bgp import *
from netbox_routing.forms.bgp import *
from netbox_routing.models.bgp import *
from netbox_routing.tables.bgp import *

__all__ = (
    'BGPSettingListView',
    'BGPSettingView',
    'BGPSettingEditView',
    'BGPSettingDeleteView',
    'BGPSettingBulkEditView',
    'BGPSettingBulkDeleteView',
    'BGPPeerTemplateListView',
    'BGPPeerTemplateView',
    'BGPPeerTemplateEditView',
    'BGPPeerTemplateDeleteView',
    'BGPPeerTemplateBulkEditView',
    'BGPPeerTemplateBulkDeleteView',
    'BGPPolicyTemplateListView',
    'BGPPolicyTemplateView',
    'BGPPolicyTemplateEditView',
    'BGPPolicyTemplateDeleteView',
    'BGPPolicyTemplateBulkEditView',
    'BGPPolicyTemplateBulkDeleteView',
    'BGPSessionTemplateListView',
    'BGPSessionTemplateView',
    'BGPSessionTemplateEditView',
    'BGPSessionTemplateDeleteView',
    'BGPSessionTemplateBulkEditView',
    'BGPSessionTemplateBulkDeleteView',
    'BGPRouterListView',
    'BGPRouterView',
    'BGPRouterEditView',
    'BGPRouterDeleteView',
    'BGPRouterBulkEditView',
    'BGPRouterBulkDeleteView',
    'BGPScopeListView',
    'BGPScopeView',
    'BGPScopeEditView',
    'BGPScopeDeleteView',
    'BGPScopeBulkEditView',
    'BGPScopeBulkDeleteView',
    'BGPAddressFamilyListView',
    'BGPAddressFamilyView',
    'BGPAddressFamilyEditView',
    'BGPAddressFamilyDeleteView',
    'BGPAddressFamilyBulkEditView',
    'BGPAddressFamilyBulkDeleteView',
)


@register_model_view(BGPAddressFamily, name='settings')
class BGPSettingViewMixin:
    child_model = BGPSetting
    table = BGPSettingTable
    filterset = BGPSettingFilterSet
    filterset_form = BGPSettingFilterForm
    actions = ObjectChildrenView.actions


#
# BGP Settings
#
@register_model_view(BGPSetting, name='list', path='', detail=False)
class BGPSettingListView(ObjectListView):
    queryset = BGPSetting.objects.all()
    filterset = BGPSettingFilterSet
    filterset_form = BGPSettingFilterForm
    table = BGPSettingTable


@register_model_view(BGPSetting)
class BGPSettingView(ObjectView):
    queryset = BGPSetting.objects.all()


@register_model_view(BGPSetting, name='add', detail=False)
@register_model_view(BGPSetting, name='edit')
class BGPSettingEditView(ObjectEditView):
    queryset = BGPSetting.objects.all()
    form = BGPSettingForm


@register_model_view(BGPSetting, name='delete')
class BGPSettingDeleteView(ObjectDeleteView):
    queryset = BGPSetting.objects.all()


@register_model_view(BGPSetting, name='bulk_edit', detail=False)
class BGPSettingBulkEditView(BulkEditView):
    queryset = BGPSetting.objects.all()
    filterset = BGPSettingFilterSet
    form = BGPSettingBulkEditForm
    table = BGPSettingTable


@register_model_view(BGPSetting, name='bulk_delete', detail=False)
class BGPSettingBulkDeleteView(BulkDeleteView):
    queryset = BGPSetting.objects.all()
    filterset = BGPSettingFilterSet
    table = BGPSettingTable


#
# BGP Templates
#
@register_model_view(BGPPeerTemplate, name='list', path='', detail=False)
class BGPPeerTemplateListView(ObjectListView):
    queryset = BGPPeerTemplate.objects.all()
    filterset = BGPPeerTemplateFilterSet
    filterset_form = BGPPeerTemplateFilterForm
    table = BGPPeerTemplateTable


@register_model_view(BGPPeerTemplate)
class BGPPeerTemplateView(ObjectView):
    queryset = BGPPeerTemplate.objects.all()


@register_model_view(BGPPeerTemplate, name='add', detail=False)
@register_model_view(BGPPeerTemplate, name='edit')
class BGPPeerTemplateEditView(ObjectEditView):
    queryset = BGPPeerTemplate.objects.all()
    form = BGPPeerTemplateForm


@register_model_view(BGPPeerTemplate, name='delete')
class BGPPeerTemplateDeleteView(ObjectDeleteView):
    queryset = BGPPeerTemplate.objects.all()


@register_model_view(BGPPeerTemplate, name='bulk_edit', detail=False)
class BGPPeerTemplateBulkEditView(BulkEditView):
    queryset = BGPPeerTemplate.objects.all()
    filterset = BGPPeerTemplateFilterSet
    form = BGPPeerTemplateBulkEditForm
    table = BGPPeerTemplateTable


@register_model_view(BGPPeerTemplate, name='bulk_delete', detail=False)
class BGPPeerTemplateBulkDeleteView(BulkDeleteView):
    queryset = BGPPeerTemplate.objects.all()
    filterset = BGPPeerTemplateFilterSet
    table = BGPPeerTemplateTable


@register_model_view(BGPPolicyTemplate, name='list', path='', detail=False)
class BGPPolicyTemplateListView(ObjectListView):
    queryset = BGPPolicyTemplate.objects.all()
    filterset = BGPPolicyTemplateFilterSet
    filterset_form = BGPPolicyTemplateFilterForm
    table = BGPPolicyTemplateTable


@register_model_view(BGPPolicyTemplate)
class BGPPolicyTemplateView(ObjectView):
    queryset = BGPPolicyTemplate.objects.all()


@register_model_view(BGPPolicyTemplate, name='add', detail=False)
@register_model_view(BGPPolicyTemplate, name='edit')
class BGPPolicyTemplateEditView(ObjectEditView):
    queryset = BGPPolicyTemplate.objects.all()
    form = BGPPolicyTemplateForm


@register_model_view(BGPPolicyTemplate, name='delete')
class BGPPolicyTemplateDeleteView(ObjectDeleteView):
    queryset = BGPPolicyTemplate.objects.all()


@register_model_view(BGPPolicyTemplate, name='bulk_edit', detail=False)
class BGPPolicyTemplateBulkEditView(BulkEditView):
    queryset = BGPPolicyTemplate.objects.all()
    filterset = BGPPolicyTemplateFilterSet
    form = BGPPolicyTemplateBulkEditForm
    table = BGPPolicyTemplateTable


@register_model_view(BGPPolicyTemplate, name='bulk_delete', detail=False)
class BGPPolicyTemplateBulkDeleteView(BulkDeleteView):
    queryset = BGPPolicyTemplate.objects.all()
    filterset = BGPPolicyTemplateFilterSet
    table = BGPPolicyTemplateTable


@register_model_view(BGPSessionTemplate, name='list', path='', detail=False)
class BGPSessionTemplateListView(ObjectListView):
    queryset = BGPSessionTemplate.objects.all()
    filterset = BGPSessionTemplateFilterSet
    filterset_form = BGPSessionTemplateFilterForm
    table = BGPSessionTemplateTable


@register_model_view(BGPSessionTemplate)
class BGPSessionTemplateView(ObjectView):
    queryset = BGPSessionTemplate.objects.all()


@register_model_view(BGPSessionTemplate, name='add', detail=False)
@register_model_view(BGPSessionTemplate, name='edit')
class BGPSessionTemplateEditView(ObjectEditView):
    queryset = BGPSessionTemplate.objects.all()
    form = BGPSessionTemplateForm


@register_model_view(BGPSessionTemplate, name='delete')
class BGPSessionTemplateDeleteView(ObjectDeleteView):
    queryset = BGPSessionTemplate.objects.all()


@register_model_view(BGPSessionTemplate, name='bulk_edit', detail=False)
class BGPSessionTemplateBulkEditView(BulkEditView):
    queryset = BGPSessionTemplate.objects.all()
    filterset = BGPSessionTemplateFilterSet
    form = BGPSessionTemplateBulkEditForm
    table = BGPSessionTemplateTable


@register_model_view(BGPSessionTemplate, name='bulk_delete', detail=False)
class BGPSessionTemplateBulkDeleteView(BulkDeleteView):
    queryset = BGPSessionTemplate.objects.all()
    filterset = BGPSessionTemplateFilterSet
    table = BGPSessionTemplateTable


#
# BGP Router Views
#
@register_model_view(BGPRouter, name='list', path='', detail=False)
class BGPRouterListView(ObjectListView):
    queryset = BGPRouter.objects.all()
    filterset = BGPRouterFilterSet
    filterset_form = BGPRouterFilterForm
    table = BGPRouterTable


@register_model_view(BGPRouter)
class BGPRouterView(ObjectView):
    queryset = BGPRouter.objects.all()
    template_name = 'netbox_routing/bgprouter.html'


@register_model_view(BGPRouter, name='add', detail=False)
@register_model_view(BGPRouter, name='edit')
class BGPRouterEditView(ObjectEditView):
    queryset = BGPRouter.objects.all()
    form = BGPRouterForm


@register_model_view(BGPRouter, name='delete')
class BGPRouterDeleteView(ObjectDeleteView):
    queryset = BGPRouter.objects.all()


@register_model_view(BGPRouter, name='bulk_edit', detail=False)
class BGPRouterBulkEditView(BulkEditView):
    queryset = BGPRouter.objects.all()
    filterset = BGPRouterFilterSet
    form = BGPRouterBulkEditForm
    table = BGPRouterTable


@register_model_view(BGPRouter, name='bulk_delete', detail=False)
class BGPRouterBulkDeleteView(BulkDeleteView):
    queryset = BGPRouter.objects.all()
    filterset = BGPRouterFilterSet
    table = BGPRouterTable


@register_model_view(BGPRouter, name='settings')
class BGPRouterSettingsView(BGPSettingViewMixin, ObjectChildrenView):
    queryset = BGPRouter.objects.all()
    tab = ViewTab(
        label='Settings',
        badge=lambda obj: BGPRouterSettingsView.child_model.objects.filter(
            router=obj
        ).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(router=parent)


@register_model_view(BGPRouter, name='peer_templates')
class BGPRouterPeerTemplatesView(ObjectChildrenView):
    queryset = BGPRouter.objects.all()
    child_model = BGPPeerTemplate
    filterset = BGPPeerTemplateFilterSet
    filterset_form = BGPPeerTemplateFilterForm
    table = BGPPeerTemplateTable
    actions = (CloneObject, EditObject, DeleteObject)
    tab = ViewTab(
        label='Peer Templates',
        badge=lambda obj: obj.peer_templates.count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(routers=parent)


@register_model_view(BGPRouter, name='policy_templates')
class BGPRouterPolicyTemplatesView(ObjectChildrenView):
    queryset = BGPRouter.objects.all()
    child_model = BGPPolicyTemplate
    filterset = BGPPolicyTemplateFilterSet
    filterset_form = BGPPolicyTemplateFilterForm
    table = BGPPolicyTemplateTable
    actions = (CloneObject, EditObject, DeleteObject)
    tab = ViewTab(
        label='Policy Templates',
        badge=lambda obj: obj.policy_templates.count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(routers=parent)


@register_model_view(BGPRouter, name='session_templates')
class BGPRouterSessionTemplatesView(ObjectChildrenView):
    queryset = BGPRouter.objects.all()
    child_model = BGPSessionTemplate
    filterset = BGPSessionTemplateFilterSet
    filterset_form = BGPSessionTemplateFilterForm
    table = BGPSessionTemplateTable
    actions = (CloneObject, EditObject, DeleteObject)
    tab = ViewTab(
        label='Session Templates',
        badge=lambda obj: obj.session_templates.count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(routers=parent)


#
# BGP Scope Views
#
@register_model_view(BGPScope, name='list', path='', detail=False)
class BGPScopeListView(ObjectListView):
    queryset = BGPScope.objects.all()
    filterset = BGPScopeFilterSet
    filterset_form = BGPScopeFilterForm
    table = BGPScopeTable


@register_model_view(BGPScope)
class BGPScopeView(ObjectView):
    queryset = BGPScope.objects.all()
    template_name = 'netbox_routing/bgpscope.html'


@register_model_view(BGPScope, name='add', detail=False)
@register_model_view(BGPScope, name='edit')
class BGPScopeEditView(ObjectEditView):
    queryset = BGPScope.objects.all()
    form = BGPScopeForm


@register_model_view(BGPScope, name='delete')
class BGPScopeDeleteView(ObjectDeleteView):
    queryset = BGPScope.objects.all()


@register_model_view(BGPScope, name='bulk_edit', detail=False)
class BGPScopeBulkEditView(BulkEditView):
    queryset = BGPScope.objects.all()
    filterset = BGPScopeFilterSet
    form = BGPScopeBulkEditForm
    table = BGPScopeTable


@register_model_view(BGPScope, name='bulk_delete', detail=False)
class BGPScopeBulkDeleteView(BulkDeleteView):
    queryset = BGPScope.objects.all()
    filterset = BGPScopeFilterSet
    table = BGPScopeTable


#
# BGP Scope Views
#
@register_model_view(BGPAddressFamily, name='list', path='', detail=False)
class BGPAddressFamilyListView(ObjectListView):
    queryset = BGPAddressFamily.objects.all()
    filterset = BGPAddressFamilyFilterSet
    filterset_form = BGPAddressFamilyFilterForm
    table = BGPAddressFamilyTable


@register_model_view(BGPAddressFamily)
class BGPAddressFamilyView(ObjectView):
    queryset = BGPAddressFamily.objects.all()
    template_name = 'netbox_routing/bgpaddressfamily.html'


@register_model_view(BGPAddressFamily, name='add', detail=False)
@register_model_view(BGPAddressFamily, name='edit')
class BGPAddressFamilyEditView(ObjectEditView):
    queryset = BGPAddressFamily.objects.all()
    form = BGPAddressFamilyForm


@register_model_view(BGPAddressFamily, name='delete')
class BGPAddressFamilyDeleteView(ObjectDeleteView):
    queryset = BGPAddressFamily.objects.all()


@register_model_view(BGPAddressFamily, name='bulk_edit', detail=False)
class BGPAddressFamilyBulkEditView(BulkEditView):
    queryset = BGPAddressFamily.objects.all()
    filterset = BGPAddressFamilyFilterSet
    form = BGPAddressFamilyBulkEditForm
    table = BGPAddressFamilyTable


@register_model_view(BGPAddressFamily, name='bulk_delete', detail=False)
class BGPAddressFamilyBulkDeleteView(BulkDeleteView):
    queryset = BGPAddressFamily.objects.all()
    filterset = BGPAddressFamilyFilterSet
    table = BGPAddressFamilyTable


@register_model_view(BGPAddressFamily, name='settings')
class BGPAddressFamilySettingsView(BGPSettingViewMixin, ObjectChildrenView):
    queryset = BGPAddressFamily.objects.all()
    tab = ViewTab(
        label='Settings',
        badge=lambda obj: BGPAddressFamilySettingsView.child_model.objects.filter(
            address_family=obj
        ).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(address_family=parent)


#
# BGP Peer Views
#
@register_model_view(BGPPeer, name='list', path='', detail=False)
class BGPPeerListView(ObjectListView):
    queryset = BGPPeer.objects.all()
    filterset = BGPPeerFilterSet
    filterset_form = BGPPeerFilterForm
    table = BGPPeerTable


@register_model_view(BGPPeer)
class BGPPeerView(ObjectView):
    queryset = BGPPeer.objects.all()


@register_model_view(BGPPeer, name='add', detail=False)
@register_model_view(BGPPeer, name='edit')
class BGPPeerEditView(ObjectEditView):
    queryset = BGPPeer.objects.all()
    form = BGPPeerForm


@register_model_view(BGPPeer, name='delete')
class BGPPeerDeleteView(ObjectDeleteView):
    queryset = BGPPeer.objects.all()


@register_model_view(BGPPeer, name='bulk_edit', detail=False)
class BGPPeerBulkEditView(BulkEditView):
    queryset = BGPPeer.objects.all()
    filterset = BGPPeerFilterSet
    form = BGPPeerBulkEditForm
    table = BGPPeerTable


@register_model_view(BGPPeer, name='bulk_delete', detail=False)
class BGPPeerBulkDeleteView(BulkDeleteView):
    queryset = BGPPeer.objects.all()
    filterset = BGPPeerFilterSet
    table = BGPPeerTable


@register_model_view(BGPPeer, name='settings')
class BGPPeerSettingsView(BGPSettingViewMixin, ObjectChildrenView):
    queryset = BGPPeer.objects.all()
    tab = ViewTab(
        label='Settings',
        badge=lambda obj: BGPPeerSettingsView.child_model.objects.filter(
            peer=obj
        ).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(peer=parent)


@register_model_view(BGPPeer, name='address-families')
class BGPPeerAddressFamiliesView(ObjectChildrenView):
    queryset = BGPPeer.objects.all()
    child_model = BGPPeerAddressFamily
    filterset = BGPPeerAddressFamilyFilterSet
    filterset_form = BGPPeerAddressFamilyFilterForm
    table = BGPPeerAddressFamilyTable
    actions = ObjectChildrenView.actions
    tab = ViewTab(
        label='Address Families',
        badge=lambda obj: BGPPeerAddressFamiliesView.child_model.objects.filter(
            peer=obj
        ).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(peer=parent)


#
# BGP Peer Address Family Views
#
@register_model_view(BGPPeerAddressFamily, name='list', path='', detail=False)
class BGPPeerAddressFamilyListView(ObjectListView):
    queryset = BGPPeerAddressFamily.objects.all()
    filterset = BGPPeerAddressFamilyFilterSet
    filterset_form = BGPPeerAddressFamilyFilterForm
    table = BGPPeerAddressFamilyTable


@register_model_view(BGPPeerAddressFamily)
class BGPPeerAddressFamilyView(ObjectView):
    queryset = BGPPeerAddressFamily.objects.all()


@register_model_view(BGPPeerAddressFamily, name='add', detail=False)
@register_model_view(BGPPeerAddressFamily, name='edit')
class BGPPeerAddressFamilyEditView(ObjectEditView):
    queryset = BGPPeerAddressFamily.objects.all()
    form = BGPPeerAddressFamilyForm


@register_model_view(BGPPeerAddressFamily, name='delete')
class BGPPeerAddressFamilyDeleteView(ObjectDeleteView):
    queryset = BGPPeerAddressFamily.objects.all()


@register_model_view(BGPPeerAddressFamily, name='bulk_edit', detail=False)
class BGPPeerAddressFamilyBulkEditView(BulkEditView):
    queryset = BGPPeerAddressFamily.objects.all()
    filterset = BGPPeerAddressFamilyFilterSet
    form = BGPPeerAddressFamilyBulkEditForm
    table = BGPPeerAddressFamilyTable


@register_model_view(BGPPeerAddressFamily, name='bulk_delete', detail=False)
class BGPPeerAddressFamilyBulkDeleteView(BulkDeleteView):
    queryset = BGPPeerAddressFamily.objects.all()
    filterset = BGPPeerAddressFamilyFilterSet
    table = BGPPeerAddressFamilyTable


@register_model_view(BGPPeerAddressFamily, name='settings')
class BGPPeerAddressFamilySettingsView(BGPSettingViewMixin, ObjectChildrenView):
    queryset = BGPPeerAddressFamily.objects.all()
    tab = ViewTab(
        label='Settings',
        badge=lambda obj: BGPPeerAddressFamilySettingsView.child_model.objects.filter(
            peer_address_families=obj
        ).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(peer_address_families=parent)


#
# BFD Profile
#
@register_model_view(BFDProfile, name='list', path='', detail=False)
class BFDProfileListView(ObjectListView):
    queryset = BFDProfile.objects.all()
    filterset = BFDProfileFilterSet
    filterset_form = BFDProfileFilterForm
    table = BFDProfileTable


@register_model_view(BFDProfile)
class BFDProfileView(ObjectView):
    queryset = BFDProfile.objects.all()


@register_model_view(BFDProfile, name='add', detail=False)
@register_model_view(BFDProfile, name='edit')
class BFDProfileEditView(ObjectEditView):
    queryset = BFDProfile.objects.all()
    form = BFDProfileForm


@register_model_view(BFDProfile, name='delete')
class BFDProfileDeleteView(ObjectDeleteView):
    queryset = BFDProfile.objects.all()


@register_model_view(BFDProfile, name='bulk_edit', detail=False)
class BFDProfileBulkEditView(BulkEditView):
    queryset = BFDProfile.objects.all()
    filterset = BFDProfileFilterSet
    form = BFDProfileBulkEditForm
    table = BFDProfileTable


@register_model_view(BFDProfile, name='bulk_delete', detail=False)
class BFDProfileBulkDeleteView(BulkDeleteView):
    queryset = BFDProfile.objects.all()
    filterset = BFDProfileFilterSet
    table = BFDProfileTable


@register_model_view(BFDProfile, name='peers')
class BFDProfilePeersView(ObjectChildrenView):
    queryset = BFDProfile.objects.all()
    child_model = BGPPeer
    table = BGPPeerTable
    filterset = BGPPeerFilterSet
    filterset_form = BGPPeerFilterForm
    actions = ObjectChildrenView.actions
    tab = ViewTab(
        label='BGP Peers',
        badge=lambda obj: BGPPeer.objects.filter(bfd=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(bfd=parent)


@register_model_view(BFDProfile, name='session_templates')
class BFDProfileSessionTemplatesView(ObjectChildrenView):
    queryset = BFDProfile.objects.all()
    child_model = BGPSessionTemplate
    table = BGPSessionTemplateTable
    filterset = BGPSessionTemplateFilterSet
    filterset_form = BGPSessionTemplateFilterForm
    actions = ObjectChildrenView.actions
    tab = ViewTab(
        label='BGP Session Template',
        badge=lambda obj: BGPSessionTemplate.objects.filter(bfd=obj).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(bfd=parent)



# Generated by Django 5.2.5 on 2026-01-17 02:48

import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0020_communities'),
    ]

    operations = [
        migrations.CreateModel(
            name='BGPSessionTemplate',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                ('bfd', models.CharField(blank=True, max_length=50, null=True)),
                ('password', models.CharField(blank=True, max_length=255, null=True)),
                (
                    'remote_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='session_templates_remote_as',
                        to='ipam.asn',
                    ),
                ),
                (
                    'local_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='session_templates_local_as',
                        to='ipam.asn',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'parent',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='children',
                        to='netbox_routing.bgpsessiontemplate',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_peer_session_templates',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Session Template',
                'verbose_name_plural': 'BGP Session Templates',
                'ordering': ('name',),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPPolicyTemplate',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'parents',
                    models.ManyToManyField(
                        related_name='children', to='netbox_routing.bgppolicytemplate'
                    ),
                ),
                (
                    'prefixlist_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_in',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'prefixlist_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_out',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'routemap_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_in',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'routemap_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_out',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_peer_policy_templates',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Policy Template',
                'verbose_name_plural': 'BGP Policy Templates',
                'ordering': ('name',),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPPeerTemplate',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'remote_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='ipam.asn',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_peer_templates',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Peer Template',
                'verbose_name_plural': 'BGP Peer Templates',
                'ordering': ('name',),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPRouter',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                (
                    'asn',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='router',
                        to='ipam.asn',
                    ),
                ),
                (
                    'assigned_object_id',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                (
                    'assigned_object_type',
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(('app_label', 'dcim'), ('model', 'region')),
                                models.Q(('app_label', 'dcim'), ('model', 'site')),
                                models.Q(
                                    ('app_label', 'dcim'), ('model', 'site_group')
                                ),
                                models.Q(('app_label', 'dcim'), ('model', 'location')),
                                models.Q(('app_label', 'dcim'), ('model', 'device')),
                                models.Q(
                                    ('app_label', 'virtualization'),
                                    ('model', 'cluster'),
                                ),
                                models.Q(
                                    ('app_label', 'virtualization'),
                                    ('model', 'virtual_machine'),
                                ),
                                _connector='OR',
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='contenttypes.contenttype',
                    ),
                ),
                (
                    'name',
                    models.CharField(
                        max_length=100,
                    ),
                ),
                (
                    'peer_templates',
                    models.ManyToManyField(
                        related_name='routers', to='netbox_routing.bgppeertemplate'
                    ),
                ),
                (
                    'policy_templates',
                    models.ManyToManyField(
                        related_name='routers', to='netbox_routing.bgppolicytemplate'
                    ),
                ),
                (
                    'session_templates',
                    models.ManyToManyField(
                        related_name='routers', to='netbox_routing.bgpsessiontemplate'
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_routers',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Router',
                'verbose_name_plural': 'BGP Routers',
                'ordering': (
                    'assigned_object_type',
                    'assigned_object_id',
                    'asn',
                    'name',
                ),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPScope',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'router',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='scopes',
                        to='netbox_routing.bgprouter',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_scopes',
                        to='tenancy.tenant',
                    ),
                ),
                (
                    'vrf',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='scopes',
                        to='ipam.vrf',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Scope',
                'verbose_name_plural': 'BGP Scopes',
                'ordering': ('router', 'vrf'),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPAddressFamily',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('address_family', models.CharField(max_length=50)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'scope',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='address_families',
                        to='netbox_routing.bgpscope',
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_address_families',
                        to='tenancy.tenant',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Address Family',
                'verbose_name_plural': 'BGP Address Families',
                'ordering': ('scope', 'address_family'),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPPeerAddressFamily',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                (
                    'assigned_object_id',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'address_family',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_address_families',
                        to='netbox_routing.bgpaddressfamily',
                    ),
                ),
                (
                    'assigned_object_type',
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgppeer'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgppeertemplate'),
                                ),
                                _connector='OR',
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='contenttypes.contenttype',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'prefixlist_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_address_families_in',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'prefixlist_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_address_families_out',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'routemap_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_address_families_in',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'routemap_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_address_families_out',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_peer_address_families',
                        to='tenancy.tenant',
                    ),
                ),
                (
                    'peer_policy',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_address_families',
                        to='netbox_routing.bgppolicytemplate',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Peer Address Family',
                'verbose_name_plural': 'BGP Peer Address Families',
                'ordering': (
                    'assigned_object_type',
                    'assigned_object_id',
                    'address_family',
                ),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPPeer',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                ('bfd', models.BooleanField(blank=True, null=True)),
                ('password', models.CharField(blank=True, max_length=255, null=True)),
                (
                    'local_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='ipam.asn',
                    ),
                ),
                (
                    'name',
                    models.CharField(
                        max_length=100,
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'peer',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='remote_peers',
                        to='ipam.ipaddress',
                    ),
                ),
                (
                    'source',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='source_peers',
                        to='ipam.ipaddress',
                    ),
                ),
                (
                    'remote_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='ipam.asn',
                    ),
                ),
                (
                    'scope',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peers',
                        to='netbox_routing.bgpscope',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='bgp_peers',
                        to='tenancy.tenant',
                    ),
                ),
                (
                    'peer_group',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peers',
                        to='netbox_routing.bgppeertemplate',
                    ),
                ),
                (
                    'peer_session',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peers',
                        to='netbox_routing.bgpsessiontemplate',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Peer',
                'verbose_name_plural': 'BGP Peers',
                'ordering': ('scope', 'peer', 'name'),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BGPSetting',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                (
                    'assigned_object_id',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                ('key', models.CharField()),
                ('value', models.CharField()),
                (
                    'assigned_object_type',
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgprouter'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgpscope'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgpaddressfamily'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgppeer'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgppeeraddressfamily'),
                                ),
                                _connector='OR',
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='contenttypes.contenttype',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Setting',
                'verbose_name_plural': 'BGP Settings',
                'ordering': ('assigned_object_type', 'assigned_object_id', 'key'),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.AddConstraint(
            model_name='bgppeertemplate',
            constraint=models.UniqueConstraint(
                fields=('name', 'remote_as'),
                name='netbox_routing_bgppeertemplate_name_remote_as',
            ),
        ),
        migrations.AddConstraint(
            model_name='bgppolicytemplate',
            constraint=models.UniqueConstraint(
                fields=('name',), name='netbox_routing_bgppolicytemplate_name'
            ),
        ),
        migrations.AddConstraint(
            model_name='bgppeeraddressfamily',
            constraint=models.UniqueConstraint(
                fields=('assigned_object_type', 'assigned_object_id', 'address_family'),
                name='netbox_routing_bgppeeraddressfamily_assigned_object_address_family',
            ),
        ),
        migrations.AddConstraint(
            model_name='bgpscope',
            constraint=models.UniqueConstraint(
                fields=('router', 'vrf'),
                name='netbox_routing_bgpscope_router_vrf',
                nulls_distinct=False,
            ),
        ),
        migrations.AddConstraint(
            model_name='bgpaddressfamily',
            constraint=models.UniqueConstraint(
                fields=('scope', 'address_family'),
                name='netbox_routing_bgpaddressfamily_scope_address_family',
            ),
        ),
        migrations.AddConstraint(
            model_name='bgpsessiontemplate',
            constraint=models.UniqueConstraint(
                fields=('name',), name='netbox_routing_bgpsessiontemplate_name'
            ),
        ),
        migrations.AddConstraint(
            model_name='bgprouter',
            constraint=models.UniqueConstraint(
                fields=('assigned_object_type', 'assigned_object_id', 'asn'),
                name='netbox_routing_bgprouter_assigned_object_asn',
                nulls_distinct=False,
            ),
        ),
        migrations.AddConstraint(
            model_name='bgprouter',
            constraint=models.UniqueConstraint(
                fields=('assigned_object_type', 'assigned_object_id', 'name'),
                name='netbox_routing_bgprouter_assigned_object_name',
                nulls_distinct=False,
            ),
        ),
        migrations.AddConstraint(
            model_name='bgppeer',
            constraint=models.UniqueConstraint(
                fields=('scope', 'peer', 'name'),
                name='netbox_routing_bgppeer_scope_peer_name',
                nulls_distinct=False,
            ),
        ),
        migrations.AddConstraint(
            model_name='bgpsetting',
            constraint=models.UniqueConstraint(
                fields=('assigned_object_type', 'assigned_object_id', 'key'),
                name='netbox_routing_bgpsettings_unique',
            ),
        ),
        migrations.AddConstraint(
            model_name='bgprouter',
            constraint=models.CheckConstraint(
                condition=models.Q(
                    ('asn__isnull', False), ('name__isnull', False), _connector='OR'
                ),
                name='netbox_routing_bgprouter_identifier',
            ),
        ),
    ]



from django import forms
from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from ipam.choices import IPAddressFamilyChoices
from ipam.forms.filtersets import PREFIX_MASK_LENGTH_CHOICES
from ipam.models import VRF, Prefix
from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.choices import AuthenticationChoices
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES, add_blank_choice
from utilities.forms.fields import (
    TagFilterField,
    DynamicModelMultipleChoiceField,
    DynamicModelChoiceField,
)
from utilities.forms.rendering import FieldSet

from netbox_routing.models import (
    EIGRPRouter,
    EIGRPAddressFamily,
    EIGRPNetwork,
    EIGRPInterface,
)

__all__ = (
    'EIGRPRouterFilterForm',
    'EIGRPAddressFamilyFilterForm',
    'EIGRPNetworkFilterForm',
    'EIGRPInterfaceFilterForm',
)


class EIGRPRouterFilterForm(NetBoxModelFilterSetForm):
    model = EIGRPRouter
    fieldsets = (FieldSet('q', 'filter_id', 'tag', 'device', 'mode'),)

    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    tag = TagFilterField(model)


class EIGRPAddressFamilyFilterForm(NetBoxModelFilterSetForm):
    model = EIGRPAddressFamily
    fieldsets = (FieldSet('q', 'filter_id', 'router_id', 'vrf_id', 'family', 'tag'),)

    router_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('EIGRP Router'),
    )
    vrf_id = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    family = forms.ChoiceField(
        required=False,
        choices=add_blank_choice(IPAddressFamilyChoices),
        label=_('Address family'),
    )
    tag = TagFilterField(model)


class EIGRPNetworkFilterForm(NetBoxModelFilterSetForm):
    model = EIGRPNetwork
    fieldsets = (
        FieldSet('q', 'filter_id', 'tag'),
        FieldSet('router_id', 'address_family', name=_('EIGRP')),
        FieldSet(
            'prefix_id',
            'vrf_id',
            'present_in_vrf_id',
            'family',
            'mask_length__lte',
            'within_include',
            'mask_length',
            name=_('Prefix'),
        ),
    )
    prefix_id = DynamicModelMultipleChoiceField(
        queryset=Prefix.objects.all(),
        required=False,
        selector=True,
        label=_('Prefix'),
    )
    router_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('EIGRP Router'),
    )
    address_family_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('EIGRP Address Family'),
    )
    vrf_id = DynamicModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        label=_('Assigned VRF'),
        null_option='Global',
    )
    present_in_vrf_id = DynamicModelChoiceField(
        queryset=VRF.objects.all(), required=False, label=_('Present in VRF')
    )
    family = forms.ChoiceField(
        required=False,
        choices=add_blank_choice(IPAddressFamilyChoices),
        label=_('Address family'),
    )
    mask_length__lte = forms.IntegerField(widget=forms.HiddenInput())
    within_include = forms.CharField(
        required=False,
        widget=forms.TextInput(
            attrs={
                'placeholder': 'Prefix',
            }
        ),
        label=_('Search within'),
    )
    mask_length = forms.MultipleChoiceField(
        required=False, choices=PREFIX_MASK_LENGTH_CHOICES, label=_('Mask length')
    )
    tag = TagFilterField(model)


class EIGRPInterfaceFilterForm(NetBoxModelFilterSetForm):
    model = EIGRPInterface
    fieldsets = (
        FieldSet('q', 'filter_id', 'tag'),
        FieldSet('router_id', 'address_family_id', name=_('EIGRP')),
        FieldSet('device_id', 'interface_id', name=_('Device')),
        FieldSet('passive', 'bfd', 'authentication', name=_('Attributes')),
    )
    router_id = DynamicModelMultipleChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=False,
        selector=True,
        label=_('EIGRP Router'),
    )
    address_family_id = DynamicModelMultipleChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        required=False,
        selector=True,
        label=_('EIGRP Address Family'),
    )
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    interface_id = DynamicModelMultipleChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        selector=True,
        label=_('Interface'),
    )
    bfd = forms.NullBooleanField(
        required=False,
        label='BFD Enabled',
        widget=forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
    )
    passive = forms.BooleanField(
        required=False,
        label='Passive Interface',
        widget=forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
    )
    authentication = forms.ChoiceField(
        choices=add_blank_choice(AuthenticationChoices), required=False
    )
    tag = TagFilterField(model)



from django.utils.translation import gettext as _

from dcim.models import Device
from ipam.models import VRF
from netbox.forms import NetBoxModelImportForm
from utilities.forms.fields import CSVModelChoiceField

from netbox_routing.models import StaticRoute

__all__ = ("StaticRouteImportForm",)


class StaticRouteImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        to_field_name="name",
        help_text=_("Name of device"),
    )
    vrf = CSVModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Name of VRF"),
    )

    class Meta:
        model = StaticRoute
        fields = (
            "name",
            "device",
            "vrf",
            "prefix",
            "next_hop",
            "metric",
            "permanent",
            "description",
            "comments",
            "tags",
        )



from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from ipam.models import VRF
from netbox.forms import NetBoxModelImportForm
from utilities.forms.fields import CSVModelChoiceField

from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface

__all__ = (
    "OSPFInstanceImportForm",
    "OSPFAreaImportForm",
    "OSPFInterfaceImportForm",
)


class OSPFInstanceImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        to_field_name="name",
        help_text=_("Name of device"),
    )
    vrf = CSVModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Name of VRF"),
    )

    class Meta:
        model = OSPFInstance
        fields = (
            "name",
            "router_id",
            "process_id",
            "device",
            "vrf",
            "description",
            "comments",
            "tags",
        )


class OSPFAreaImportForm(NetBoxModelImportForm):

    class Meta:
        model = OSPFArea
        fields = (
            "area_id",
            "area_type",
            "description",
            "comments",
            "tags",
        )


class OSPFInterfaceImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
        to_field_name="name",
        help_text=_("Name of device"),
    )
    instance = CSVModelChoiceField(
        queryset=OSPFInstance.objects.all(),
        required=True,
        to_field_name="name",
        help_text=_("Name of OSPF Instance"),
    )
    area = CSVModelChoiceField(
        queryset=OSPFArea.objects.all(),
        required=True,
        to_field_name="name",
        help_text=_("Area ID"),
    )
    interface = CSVModelChoiceField(
        queryset=Interface.objects.all(),
        required=True,
        to_field_name="name",
        help_text=_("Name of interface"),
    )

    class Meta:
        model = OSPFInterface
        fields = (
            "device",
            "instance",
            "area",
            "interface",
            "passive",
            "priority",
            "bfd",
            "authentication",
            "passphrase",
            "description",
            "comments",
            "tags",
        )



import uuid
from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils.translation import gettext as _

from netbox.context_managers import event_tracking
from netbox_routing.models.bgp import *
from netbox_routing.models.community import *
from netbox_routing.models.objects import *
from users.models import User
from utilities.request import NetBoxFakeRequest


def resolve_assigned_object(item):
    if getattr(item, 'device', None):
        return item.device
    elif getattr(item, 'virtualmachine', None):
        return item.virtualmachine
    elif getattr(item, 'site', None):
        return item.site
    return None


def pks_from_through_relationship(item, column):
    local_class = item.__class__
    local_column = getattr(local_class, column)
    remote_class = local_column.rel.related_model

    local_field = f'{local_class._meta.model_name}_id'
    remote_field = f'{remote_class._meta.model_name}_id'

    filter = {f'{local_field}': item.pk}

    pks = []
    for row in local_column.through.objects.filter(**filter):
        pks.append(getattr(row, f'{remote_field}'))

    return pks


class Command(BaseCommand):
    # def add_arguments(self, parser):
    #     parser.add_argument('--plugin-version', dest='version', help="Version of Netbox-BGP")

    def handle(self, *args, **options):
        try:
            user = User.objects.get(username='NetBox BGP Migration')
        except User.DoesNotExist:
            user = User(username='NetBox BGP Migration')
            user.save()

        request = NetBoxFakeRequest(
            {
                'META': {},
                'POST': {},
                'GET': {},
                'FILES': {},
                'user': user,
                'path': '',
                'id': uuid.uuid4(),
            }
        )

        from netbox_routing.models import netbox_bgp

        try:
            with transaction.atomic(), event_tracking(request):
                mapping = {
                    'community': {},
                    'community_list': {},
                    'community_list_entry': {},
                    'prefix_list': {},
                    'prefix_list_entry': {},
                    'aspath': {},
                    'aspath_entry': {},
                    'route_map': {},
                    'route_map_entry': {},
                    'router': {},
                    'peer_group': {},
                }

                local_as_default = None
                if sess := netbox_bgp.BGPSession.objects.filter(
                    device__isnull=True, site__isnull=True, local_as__isnull=False
                ).first():
                    local_as_default = sess.local_as
                elif sess := netbox_bgp.BGPSession.objects.filter(
                    site__isnull=True, local_as__isnull=False
                ).first():
                    local_as_default = sess.local_as
                elif sess := netbox_bgp.BGPSession.objects.filter(
                    local_as__isnull=False
                ).first():
                    local_as_default = sess.local_as

                if not local_as_default:
                    raise Exception(
                        _(
                            'You must have at least one session with an ASN defined to allow the creation of a '
                            'default router'
                        )
                    )

                router = BGPRouter(
                    name='default',
                    asn=local_as_default,
                    description='Default Router',
                )
                router.full_clean()
                router.save()
                mapping['router'][None] = router

                scope = BGPScope(
                    router=router,
                    description='Default Scope',
                )
                scope.full_clean()
                scope.save()

                af = BGPAddressFamily(
                    scope=scope,
                    address_family='ipv4-unicast',
                    description='Default v4 AF',
                )
                af.full_clean()
                af.save()

                af = BGPAddressFamily(
                    scope=scope,
                    address_family='ipv6-unicast',
                    description='Default v6 AF',
                )
                af.full_clean()
                af.save()

                for item in netbox_bgp.BGPSession.objects.all():
                    if not item.local_as:
                        raise Exception(_('All BGP Sessions require a local ASN'))

                    assigned_object = item.device if item.device else item.site
                    if assigned_object in mapping['router'].keys():
                        continue

                    try:
                        if item.device:
                            router = BGPRouter.objects.get(
                                device=assigned_object, asn=item.asn
                            )
                        elif item.site:
                            router = BGPRouter.objects.get(site=item.site, asn=item.asn)
                        else:
                            router = BGPRouter.objects.get(asn=item.asn)
                    except BGPRouter.DoesNotExist:
                        router = BGPRouter(
                            assigned_object=assigned_object,
                            asn=item.asn,
                        )
                        router.full_clean()
                        router.save()
                        mapping['router'][assigned_object] = router
                        if not mapping['router'].get(item.pk):
                            mapping['router'][item.pk] = router

                    try:
                        scope = BGPScope.objects.get(router=router)
                    except BGPScope.DoesNotExist:
                        scope = BGPScope(
                            router=router,
                        )
                        scope.full_clean()
                        scope.save()

                    try:
                        af = BGPAddressFamily.objects.get(
                            scope=scope, address_family='ipv4-unicast'
                        )
                    except BGPAddressFamily.DoesNotExist:
                        af = BGPAddressFamily(
                            scope=scope,
                            address_family='ipv4-unicast',
                            description='Default AF',
                        )
                        af.full_clean()
                        af.save()

                    try:
                        af = BGPAddressFamily.objects.get(
                            scope=scope, address_family='ipv6-unicast'
                        )
                    except BGPAddressFamily.DoesNotExist:
                        af = BGPAddressFamily(
                            scope=scope,
                            address_family='ipv6-unicast',
                            description='Default AF',
                        )
                        af.full_clean()
                        af.save()

                for item in netbox_bgp.Community.objects.all():
                    try:
                        entry = Community.objects.get(community=item.value)
                        if not mapping['community'].get(item.pk):
                            mapping['community'][item.pk] = entry
                    except Community.DoesNotExist:
                        entry = Community(
                            community=item.value,
                            status=item.status,
                            role=item.role,
                            tenant=item.tenant,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['community'][item.pk] = entry

                for item in netbox_bgp.CommunityList.objects.all():
                    try:
                        entry = CommunityList.objects.get(name=item.name)
                        if not mapping['community_list'].get(item.pk):
                            mapping['community_list'][item.pk] = entry
                    except CommunityList.DoesNotExist:
                        entry = CommunityList(
                            name=item.name,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['community_list'][item.pk] = entry

                for item in netbox_bgp.CommunityListRule.objects.all():
                    community = mapping['community'][item.community.pk]
                    community_list = mapping['community_list'][item.community_list.pk]

                    try:
                        entry = CommunityListEntry.objects.get(
                            community_list=community_list, community=community
                        )
                        if not mapping['community_list_entry'].get(item.pk):
                            mapping['community_list_entry'][item.pk] = entry
                    except CommunityListEntry.MultipleObjectsReturned:
                        for entry in CommunityListEntry.objects.filter(
                            community_list=community_list, community=community
                        ):
                            if not mapping['community_list_entry'].get(item.pk):
                                mapping['community_list_entry'][item.pk] = entry
                    except CommunityListEntry.DoesNotExist:
                        entry = CommunityListEntry(
                            community_list=community_list,
                            action=item.action,
                            community=community,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['community_list_entry'][item.pk] = entry

                for item in netbox_bgp.PrefixList.objects.all():
                    family = 4 if item.family == 'ipv4' else 6
                    try:
                        entry = PrefixList.objects.get(name=item.name)
                        if not mapping['prefix_list'].get(item.pk):
                            mapping['prefix_list'][item.pk] = entry
                    except PrefixList.DoesNotExist:
                        entry = PrefixList(
                            name=item.name,
                            family=family,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['prefix_list'][item.pk] = entry

                for item in netbox_bgp.PrefixListRule.objects.all():
                    prefix_list = mapping['prefix_list'][item.prefix_list.pk]
                    prefix = None
                    if item.prefix:
                        prefix = item.prefix
                    elif item.prefix_custom:
                        try:
                            prefix = CustomPrefix.objects.get(prefix=item.prefix_custom)
                        except CustomPrefix.DoesNotExist:
                            prefix = CustomPrefix(prefix=item.prefix_custom)
                            prefix.clean()
                            prefix.save()

                    try:
                        entry = PrefixListEntry.objects.get(
                            prefix_list=prefix_list, sequence=item.index
                        )
                        if not mapping['prefix_list_entry'].get(item.pk):
                            mapping['prefix_list_entry'][item.pk] = entry
                    except PrefixListEntry.DoesNotExist:
                        entry = PrefixListEntry(
                            prefix_list=prefix_list,
                            sequence=item.index,
                            action=item.action,
                            prefix=prefix,
                            ge=item.ge,
                            le=item.le,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['prefix_list_entry'][item.pk] = entry

                for item in netbox_bgp.ASPathList.objects.all():
                    try:
                        entry = ASPath.objects.get(name=item.name)
                        if not mapping['aspath'].get(item.pk):
                            mapping['aspath'][item.pk] = entry
                    except ASPath.DoesNotExist:
                        entry = ASPath(
                            name=item.name,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['aspath'][item.pk] = entry

                for item in netbox_bgp.ASPathListRule.objects.all():
                    aspath = mapping['aspath'][item.aspath_list.pk]
                    try:
                        entry = ASPathEntry.objects.get(
                            aspath=aspath, sequence=item.index
                        )
                        if not mapping['aspath_entry'].get(item.pk):
                            mapping['aspath_entry'][item.pk] = entry
                    except ASPathEntry.DoesNotExist:
                        entry = ASPathEntry(
                            aspath=aspath,
                            sequence=item.index,
                            action=item.action,
                            pattern=item.pattern,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['aspath_entry'][item.pk] = entry

                for item in netbox_bgp.RoutingPolicy.objects.all():
                    #
                    # Weight is discarded
                    #
                    try:
                        entry = RouteMap.objects.get(name=item.name)
                        if not mapping['route_map'].get(item.pk):
                            mapping['route_map'][item.pk] = entry
                    except RouteMap.DoesNotExist:
                        entry = RouteMap(
                            name=item.name,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['route_map'][item.pk] = entry

                for item in netbox_bgp.RoutingPolicyRule.objects.all():
                    route_map = mapping['route_map'][item.routing_policy.pk]
                    community_list = [
                        mapping['community_list'][cl].pk
                        for cl in item.match_community_list.values_list(
                            'communitylist_id', flat=True
                        )
                    ]
                    community = [
                        mapping['community'][c].pk
                        for c in item.match_community.values_list(
                            'community_id', flat=True
                        )
                    ]
                    aspath = [
                        mapping['aspath'][asp].pk
                        for asp in item.match_aspath_list.values_list(
                            'aspathlist_id', flat=True
                        )
                    ]
                    prefix_list = [
                        mapping['prefix_list'][pl].pk
                        for pl in item.match_ip_address.values_list(
                            'prefixlist_id', flat=True
                        )
                    ]
                    prefix_list.extend(
                        [
                            mapping['prefix_list'][pl].pk
                            for pl in item.match_ipv6_address.values_list(
                                'prefixlist_id', flat=True
                            )
                        ]
                    )
                    match = item.match_custom
                    if not match:
                        match = {}
                    try:
                        entry = RouteMapEntry.objects.get(
                            route_map=route_map, sequence=item.index
                        )
                        if not mapping['route_map_entry'].get(item.pk):
                            mapping['route_map_entry'][item.pk] = entry
                    except RouteMapEntry.DoesNotExist:
                        entry = RouteMapEntry(
                            route_map=route_map,
                            sequence=item.index,
                            action=item.action,
                            match=match,
                            set=item.set_actions,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        entry.match_prefix_list.set(prefix_list)
                        entry.match_community_list.set(community_list)
                        entry.match_community.set(community)
                        entry.match_aspath.set(aspath)
                        mapping['route_map_entry'][item.pk] = entry

                for item in netbox_bgp.BGPPeerGroup.objects.all():

                    if (
                        item.import_policies.count() > 1
                        or item.import_policies.count() > 1
                    ):
                        error = (
                            f'Only 1 import or export policy is supported per Peer Template.  There are '
                            f'{item.import_policies.count()} Import Policies and {item.export_policies.count()} Export'
                            f' Policies. Only the first policy per type will be applied.'
                        )
                        # raise Exception(_(error))
                        print(error)

                    router = mapping['router'][None]
                    scope = router.scopes.first()
                    ipv4_af = scope.address_families.filter(
                        address_family='ipv4-unicast'
                    ).first()
                    ipv6_af = scope.address_families.filter(
                        address_family='ipv6-unicast'
                    ).first()
                    try:
                        entry = BGPPeerTemplate.objects.get(name=item.name)
                        if not mapping['peer_group'].get(item.pk):
                            mapping['peer_group'][item.pk] = entry
                    except BGPPeerTemplate.DoesNotExist:
                        entry = BGPPeerTemplate(
                            name=item.name,
                            description=item.description,
                            comments=item.comments,
                        )
                        entry.full_clean()
                        entry.save()
                        mapping['peer_group'][item.pk] = entry

                    import_policies = (
                        mapping['route_map'][
                            item.import_policies.first().routingpolicy.pk
                        ]
                        if item.import_policies
                        else None
                    )
                    export_policies = (
                        mapping['route_map'][
                            item.export_policies.first().routingpolicy.pk
                        ]
                        if item.export_policies
                        else None
                    )

                    try:
                        peer_af = BGPPeerAddressFamily.objects.get(
                            peer_group=entry, address_family=ipv4_af
                        )
                    except BGPPeerAddressFamily.DoesNotExist:
                        peer_af = BGPPeerAddressFamily(
                            assigned_object=entry,
                            address_family=ipv4_af,
                            routemap_in=import_policies,
                            routemap_out=export_policies,
                        )
                        peer_af.full_clean()
                        peer_af.save()

                    try:
                        peer_af = BGPPeerAddressFamily.objects.get(
                            peer_group=entry, address_family=ipv6_af
                        )
                    except BGPPeerAddressFamily.DoesNotExist:
                        peer_af = BGPPeerAddressFamily(
                            assigned_object=entry,
                            address_family=ipv6_af,
                            routemap_in=import_policies,
                            routemap_out=export_policies,
                        )
                        peer_af.full_clean()
                        peer_af.save()

                for item in netbox_bgp.BGPSession.objects.all():
                    if (
                        item.import_policies.count() > 1
                        or item.import_policies.count() > 1
                    ):
                        error = (
                            f'Only 1 import or export policy is supported per Peer Session.  There are '
                            f'{item.import_policies.count()} Import Policies and {item.export_policies.count()} Export'
                            f' Policies. Only the first policy per type will be applied.'
                        )
                        # raise Exception(_(error))
                        print(error)
                    assigned_object = resolve_assigned_object(item)
                    router = mapping['router'][assigned_object]
                    scope = router.scopes.first()

                    if item.local_address.address.version == 4:
                        af = scope.address_families.filter(
                            address_family='ipv4-unicast'
                        ).first()
                    else:
                        af = scope.address_families.filter(
                            address_family='ipv6-unicast'
                        ).first()
                    try:
                        peer = BGPPeer.objects.get(
                            scope=scope, peer=item.remote_address, name=item.name
                        )
                    except BGPPeer.DoesNotExist:
                        peer = BGPPeer(
                            name=item.name,
                            scope=scope,
                            source=item.local_address,
                            peer=item.remote_address,
                            remote_as=item.remote_as,
                            local_as=item.local_as,
                            peer_group=(
                                mapping['peer_group'].get(item.peer_group.pk)
                                if item.peer_group
                                else None
                            ),
                            enabled=True if item.status == 'active' else False,
                            tenant=item.tenant,
                            description=item.description,
                            comments=item.comments,
                        )
                        peer.full_clean()
                        peer.save()

                    import_policy = None
                    export_policy = None
                    if item.import_policies:
                        policy = item.import_policies.first()
                        import_policy = mapping['route_map'].get(
                            policy.routingpolicy.pk
                        )
                    if item.export_policies:
                        policy = item.export_policies.first()
                        export_policy = mapping['route_map'].get(
                            policy.routingpolicy.pk
                        )

                    try:
                        peeraf = BGPPeerAddressFamily.objects.get(
                            peer=peer, address_family=af
                        )
                    except BGPPeerAddressFamily.DoesNotExist:
                        peeraf = BGPPeerAddressFamily(
                            assigned_object=peer,
                            address_family=af,
                            routemap_in=import_policy,
                            routemap_out=export_policy,
                            prefixlist_in=(
                                mapping['prefix_list'].get(item.prefix_list_in.pk)
                                if item.prefix_list_in
                                else None
                            ),
                            prefixlist_out=(
                                mapping['prefix_list'].get(item.prefix_list_out.pk)
                                if item.prefix_list_out
                                else None
                            ),
                        )
                        peeraf.full_clean()
                        peeraf.save()
                # raise Exception('Finished')
        except Exception as e:
            print(f'Last item: {item} ({type(item)})')
            print(f'Exception encountered: {e}; rolling back migration')
            raise e

        user.delete()



from django.test import TestCase

from tenancy.models import Tenant

from netbox_routing.choices import ActionChoices
from netbox_routing.forms.community import *

__all__ = ('CommunityTestCase', 'CommunityListTestCase', 'CommunityListEntryTestCase')


class CommunityTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.tenant = Tenant.objects.create(name='Tenant 1')
        cls.role = Role.objects.create(name='Role 1')

    def test_community(self):
        form = CommunityForm(
            data={
                'community': '64512',
                'role': self.role,
                'status': 'active',
                'tenant': self.tenant,
                'description': 'Description',
                'comment': 'Comment',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_community_minimal(self):
        form = CommunityForm(
            data={
                'community': '64512',
                'status': 'active',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid_community(self):
        form = CommunityForm(
            data={
                'community': 'g443gf',
                'status': 'active',
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()

    def test_form_invalid_status(self):
        form = CommunityForm(
            data={
                'community': '64512:21333',
                'status': 'private',
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()


class CommunityListTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.tenant = Tenant.objects.create(name='Tenant 1')

    def test_community_list(self):
        form = CommunityListForm(
            data={
                'name': 'Community List 1',
                'tenant': self.tenant,
                'description': 'Description',
                'comment': 'Comment',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = CommunityListForm(
            data={
                'tenant': self.tenant,
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()
            form.save()


class CommunityListEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.tenant = Tenant.objects.create(name='Tenant 1')
        cls.role = Role.objects.create(name='Role 1')
        cls.community = Community.objects.create(
            community='64512',
            status='active',
            role=cls.role,
            tenant=cls.tenant,
        )
        cls.community_list = CommunityList.objects.create(
            name='Community List 1',
            tenant=cls.tenant,
        )

    def test_community_list_entry(self):
        form = CommunityListEntryForm(
            data={
                'community_list': self.community_list.pk,
                'community': self.community.pk,
                'action': ActionChoices.PERMIT,
                'description': 'Description',
                'comment': 'Comment',
            }
        )
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def test_form_invalid(self):
        form = CommunityListEntryForm(
            data={
                'community': self.community.pk,
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()

        form = CommunityListEntryForm(
            data={
                'community_list': self.community_list.pk,
            }
        )
        self.assertFalse(form.is_valid())
        with self.assertRaises(ValueError):
            form.save()



import django_filters
import netaddr
from django.db.models import Q

from dcim.models import Device
from ipam.models import VRF
from netbox.filtersets import NetBoxModelFilterSet
from utilities.filtersets import register_filterset

from netbox_routing.models import StaticRoute


@register_filterset
class StaticRouteFilterSet(NetBoxModelFilterSet):

    device = django_filters.ModelMultipleChoiceFilter(
        field_name='devices__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device (name)',
    )
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='devices',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    vrf = django_filters.ModelMultipleChoiceFilter(
        field_name='vrf__name',
        queryset=VRF.objects.all(),
        to_field_name='name',
        label='VRF (name)',
    )
    vrf_id = django_filters.ModelMultipleChoiceFilter(
        field_name='vrf',
        queryset=VRF.objects.all(),
        label='VRF (ID)',
    )

    prefix = django_filters.CharFilter(
        method='filter_prefix',
        label='Prefix',
    )

    next_hop = django_filters.CharFilter(
        method='filter_address',
        label='Prefix',
    )

    class Meta:
        model = StaticRoute
        fields = (
            'name',
            'devices',
            'device',
            'device_id',
            'vrf',
            'vrf_id',
            'prefix',
            'metric',
            'next_hop',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(devices__name__icontains=value)
            | Q(vrf__name__icontains=value)
            | Q(vrf__rd__icontains=value)
            | Q(prefix__icontains=value)
            | Q(next_hop__icontains=value)
            | Q(name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()

    def filter_prefix(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(**{f'{name}': query})
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()

    def filter_address(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = netaddr.IPAddress(value)
            return queryset.filter(**{f'{name}': query})
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()
